; File Name   :	bcs33.bin
; Base Address:	0000h Range: 0000h - 1000h Loaded length: 1000h
; fr Arnold-Assembler AS

		cpu	Z80undoc

;-----------------------------------------------------------------------------
; AS-Funktionen
;-----------------------------------------------------------------------------

; obere 8 Bit: hi(CONBU)
hi              function x,(x>>8)&255

; untere 8 Bit: lo(CONBU)
lo              function x, x&255


;-----------------------------------------------------------------------------
;
; S/P-BASIC INTERPRETER FUER HEIMCOMPUTER BCS3 (4 KBYTE ROM, MIND. 1 KB RAM)
; **************************************************************************
;
; BEARBEITET VON FRANK PRUEFER AUF DER GRUNDLAGE
; VON BASIC-SE 3.1, AUTOR ECKHARD SCHILLER (RFE HEFT 9/1986)
; VARIANTE FUER MAX. 28 BILDSCHIRMZEILEN MIT (29)30 ZEICHEN, 2,5 MHZ CPU-TAKT
;
; VERSION 3.2: STAND 20.01.1987
;
; Neu reassembliert und an die Syntax des Arnold-Assembler AS angepat von
; Volker Pohlers, Mrz 2009;
; kommentiert und Einbau der nderungen fr Version 3.3 von Frank Prfer
;
; Die Kommentare wurden 1:1 aus handschriftlichen Notizen von Frank Prfer
; aus den Jahren 1986 und 1987 (niedergeschrieben whrend der Erforschung
; der Interna von Version 3.1 und der Entwicklung der ersten Vorstufen der
; Version 3.2) in diese Datei bernommen.
; Die erzeugte Binrdatei ist byteidentisch mit dem lauffhigen Original!
;
;
; UPDATE AUF VERSION 3.3 - Stand Frhjahr 1987
; ======================
; Nach Abschlu der Entwicklung der Version 3.2 wurden noch an (teilweise
; versteckten) Stellen einzelne Optimierungsmglichkeiten erkannt, die es
; erlaubten, weitere insgesamt 18 Byte im ROM "freizuschaufeln". Der
; dadurch gewonnene Platz wurde verwendet, um das Verhalten von "IF" im
; folgenden Sinne zu ndern: Bei nicht erfllter Bedingung wird der Rest
; der BASIC-ZEILE bersprungen, nicht nur wie bisher die nchste Anweisung.
; (Siehe unten Symbol "REM:" - bei V3.2 kam sinngem "NANWS:" zum Einsatz.)
;
; Diese Version wurde etwa im Mai 1987 auf einem Mikrorechner-Entwicklungs-
; system (MRES) assembliert und damals auch praktisch in einem BCS3 erprobt.
; Im Unterschied zur Version 3.2 existiert 2009 jedoch kein Assembler- bzw.
; Hexadezimallisting mehr, sondern es gibt nur noch handschriftliche Notizen
; der nderungen, die 1987 in ein MRES-Listing der V3.2 eingetragen worden
; waren. Diese wurden 1:1 in diese Quelldatei bernommen - allerdings kann
; die sich daraus ergebende Binrdatei aus o.g. Grnden nicht mehr mit dem
; Original von 1987 unmittelbar byteweise verglichen werden. Da das Ergebnis
; jedoch im Emulator genau wie vorgesehen funktioniert, ist mit an Sicherheit
; grenzender Wahrscheinlichkeit davon auszugehen, da die erzeugte Binrdatei
; der seinerzeit erstellten Version entspricht.
;
;-----------------------------------------------------------------------------

RAM		equ	3C00h		; log. RAM-Beginn, BWS mu im 1. KByte
					; liegen, anschlieend Arbeitsspeicher

BZLEN		equ	30		; **** Zeilenlnge Bildschirm inkl. Endezeichen
NL		equ	1Eh		; Zeichen New Line


		ORG 0

RST0:		ld	hl, IWCTC	; Tabelle Init-Werte
		ld	bc, 3F8h	; CTC 0
		otir
		ld	bc, 2F9h	; CTC 1
		otir
		ld	bc, 2FAh	; CTC 2
		otir
		ld	bc, 2FBh	; CTC 3
		otir
		ld	sp, BWS		; Beginn Bildwiederholspeicher
		im	2
		jp	BASIC		; Beginn Interpreter

; Initialisierungskonstanten CTC

IWCTC:		db 00000101b		; CTC 0, Zeitgeber, Vorteiler 16
		db 10			; Zeitkonstante: 160 Takte fr Zeilenimpuls
		db Lo(RST28)		; das ist IVTAB, nur fr CTC 1 genutzt
		db 11111111b		; CTC 1, Zhler, Vorteiler 256, Trigger, Reset,	Interrupt
		db 64			; Zeitkonstante: Interrupt aller 64 Fernsehzeilen
		db 01111111b		; CTC 2, Zhler, Vorteiler 256,	Trigger, Reset
		db 5			; Zeitkonstante: 5*64 = 320 FS-Zeilen: Halbbild-Austastimpuls
		db 01111111b		; CTC 3, Zhler, Vorteiler 256,	Trigger, Reset
		db 50			; Zeitkonstante: 50 Halbbilder = 1-Sekunden-Takt (redundant)

		ORG 28h

RST28:		jr	ZAUS		; Zeichenausgabe

		dw BSINT		; Int-Adresse CTC 1 (Bildschirmsteuerung)

BSIBZ:		pop	de		; Ende Fernsehzeile
		inc	de		; Leerbefehl
		jp	(hl)		; Sprung in den BWS

		rst	38h		; frei

		ORG 30h

RST30:		pop	de		; Ende Bildschirminterrupt (Opcode F7h)
		exx
BSIE:		ex	af, af'
		ei
		reti

; Sperren Bildschirmausgabe (beschleunigt Arbeit von rechenintensiven Programmen)

BSSP:		di
		ret

		ORG 38h			; Ende Bildschirmzeile (Opcode FFh)

RST38:		djnz	BSIBZ		; bis alle 8 Fernsehzeilen durch
		ld	b, a		; B=8: FS-Zeilenzahl
		ld	c, 0		; Leerbefehl
		pop	hl		; Anfang nchster Zeile aus Stack holen
		jp	(hl)		; Sprung in den BWS


		db 0FFh			; frei fr Ergnzungen 3,5-MHz-Version
		db 0FFh
		db 0FFh
		db 0FFh


;------------------------------------------------------------------------------
; Bildschirmsteuer-Interrupt
;------------------------------------------------------------------------------

BSINT:		ex	af, af'
		in	a, (0FAh)	; CTC-Kanal 2 lesen
		cp	4		; 64 leere Fernsehzeilen oberer Rand
		jr	nz, BSIE
		ld	(1400h), a	; Auslsen WAIT	bis Zeilenanfang
		exx
		add	a, a
		ld	b, a		; B=8= FS-Zeilenzahl/Zeichenzeile
		ld	hl, BWS-0400H	; korrigierte Anfangsadresse BWS
		nop			; Fllbefehl: linker BS-Rand 1 Zeichen
		jp	(hl)		; Sprung in den BWS

		db 0FFh			; frei fr Ergnzungen 3,5 MHz-Version

;------------------------------------------------------------------------------
; Bildschirm-Lschung
;------------------------------------------------------------------------------

BSCL:		ld	bc, (ZZ)	; Zeilenzahl Bildschirm
BSCL1:		ld	hl, BWS		; Beginn Bildwiederholspeicher
		ld	(CPOS),	hl	; Bildschirmpointer
BSCL2:		ld	b, BZLEN-1	; Anzahl sichtbarer Zeichen/Zeile
BSCL3:		ld	(hl), ' '
		jr	BSCL4		; NMI (ROM-Adr. 0066h) berspringen

		ORG 66h

NMI:		jr	RST0		; NMI-Sprung (wie RESET)

BSCL4:		inc	hl		; Zeile lschen
		djnz	BSCL3
		ld	(hl), 0FFh	; Opcode "RST 38h" eintragen: Zeilenende
		inc	hl
		dec	c
		jr	nz, BSCL2	; bis alle Zeilen durch
		ld	(hl), 0F7h	; Opcode "RST 30h" eintragen: BWS-Ende
		ret

;------------------------------------------------------------------------------
; Ausgabe eines	Zeichens aus A auf den Bildschirm
;------------------------------------------------------------------------------

ZAUS:		push	af		; Einsprung vom	RST 28H
		push	hl
		ld	hl, (CPOS)	; Bildschirmpointer
		cp	NL
		jr	z, ZAUS6
		and	7Fh		; nur 7-Bit-Zeichen zulssig
		ld	(hl), a
		inc	hl
ZAUS1:		ld	a, (hl)
		cp	0F7h		; Bildschirmende?
		jr	nz, ZAUS4	; nein
		push	de
		push	bc
		ld	hl, (BZ)	; Anfang Eingabezeile im BWS
		ld	de, BZLEN
		xor	a
		sbc	hl, de		; eine Zeile zurck wegen anschlieendem Rollen
		ld	(BZ), hl	; Anfang Eingabezeile im BWS
		ld	bc, (ZZ-1)	; B = Bildschirm-Zeilenzahl
		ld	h, a
		ld	l, a		; HL = 0
		dec	b
ZAUS2:		add	hl, de		; HL wird BWS-Lnge minus 1 Zeile
		djnz	ZAUS2
		ld	b, h
		ld	c, l
		dec	bc		; BC = BWS-Lnge minus 1 Zeile minus 1
		ld	hl, BWS+BZLEN	; Quelle Umspeichern: 2. Zeile
		ld	de, BWS		; Ziel Umspeichern: Beginn BWS
		ldir			; nach oben Rollen
		ld	b, BZLEN-1
ZAUS3:		dec	hl
		ld	(hl), ' '	; letzte Zeile lschen
		djnz	ZAUS3
		pop	bc
		pop	de
ZAUS4:		bit	7, (hl)
		jr	z, ZAUS5	; kein Zeilenendezeichen
		inc	hl		; Zeilenende FFH berlesen
		jr	ZAUS1
ZAUS5:		ld	(CPOS),	hl	; Bildschirmpointer auf nchstes Zeichen
		pop	hl
		pop	af
		ret
ZAUS6:		bit	7, (hl)
		inc	hl
		jr	z, ZAUS6	; Anfang nchster Zeile suchen
		jr	ZAUS1

;------------------------------------------------------------------------------
; Eingabe eines	Zeichens von der Tastatur
;------------------------------------------------------------------------------

ZEIN:		push	hl
		push	bc
ZEIN1:		ld	b, 4		; *** Zahl Entprellungszyklen (2...6)
ZEIN2:		call	TSTS		; Tastaturstatus
		jr	nz, ZEIN1	; wenn gedrckt: auf Loslassen warten
		djnz	ZEIN2		; entspr. Entprellungszyklen ungedrckt abfragen
ZEIN3:		call	TSTS		; Tastaturstatus
		jr	z, ZEIN3	; wenn ungedrckt
		ld	c, a		; Tastencode
		call	TSTS		; Tastaturstatus nochmal abfragen
		cp	c		; Code wie vorher?
		jr	nz, ZEIN3	; entprellen, wenn nein
		pop	bc
		pop	hl
		ret

;------------------------------------------------------------------------------
; Tastaturstatus
;------------------------------------------------------------------------------

TSTS:		push	bc
		ld	hl, 13FEh	; Bit0 = 0 : Zeile 1 adressieren
		ld	c, 0		; Zeilennummer = 0
TSTS1:		ld	a, (hl)
		cp	(hl)
		jr	nz, TSTS1	; entprellen
		ld	b, 0		; Spaltennummer = 0
TSTS2:		bit	3, a
		jr	nz, TSTS3	; Spalte(1) gedrckt erkannt
		inc	b		; Spaltennummer erhhen
		add	a, a		; Linksverschiebung bis Spalte gefunden
		jr	nz, TSTS2	; wenn Taste gedrckt
		inc	c		; Zeilennummer erhhen
		scf
		rl	l
		rl	h		; Zeilenadresse links verschieben
		ld	a, h
		cp	23h		; wenn alt Bit10=0 wre
		jr	z, TSTS7	; alle 10 Zeilen durch
		and	3		; nur Bit 8 und 9 gltig
		add	a, 10h		; Bit12=1 (Tastaturadr.bereich 1000h-13ffh)
		ld	h, a
		jr	TSTS1
TSTS3:		ld	a, (11FFh)	; Bit9=0 - Zeile 10 adressieren
		bit	0, a		; SHIFT gedrckt?
		jr	z, TSTS4	; nein
		set	2, b		; Shift-Kennung setzen (Spalte + 4)
TSTS4:		ld	a, b		; A = Spaltennummer
		ld	hl, TASTB	; "0AKU	&:"
		add	a, l		; Adresse des Zeichens in Tab. ermitteln
		ld	l, a
		ld	a, (hl)		; Spaltenanfang
		add	a, c		; Zeile
		cp	0Ah
		jr	c, TSTS7	; Spalte "U..."+SHIFT: gilt als ungedrckt
		cp	5Eh
		jr	z, TSTS7	; SHIFT allein: gilt als ungedrckt
		cp	5Dh		; <ENTER>?
		jr	nz, TSTS5	; nein
		ld	a, 7Fh		; ENTER-Zeichen
TSTS5:		cp	5Bh		; Leerzeichen?
		jr	nz, TSTS6
		sub	3Bh		; "Z"+1 ---> Leerzeichen
TSTS6:		pop	bc
		ret
TSTS7:		sub	a		; keine Taste gedrckt
		pop	bc
		ret

;------------------------------------------------------------------------------
; Eingabe einer	Zeichenkette (Zeile)
;------------------------------------------------------------------------------

ZLEIN:		ld	hl, (CPOS)	; Bildschirmpointer
		ld	(BZ), hl	; Anfang Eingabezeile(n) im BWS
ZLEIN1:		ld	(hl), '_'	; Kursor im BWS
		call	ZEIN		; Zeichen holen
		rst	28h		; Echoausg.
		ld	hl, (CPOS)	; Bildschirmpointer
		cp	5Ch		; Backspace?
		jr	nz, ZLEIN4
		dec	hl
		bit	7, (hl)
		jr	z, ZLEIN2	; kein Zeilenende
		dec	hl		; Zeilenende belassen
ZLEIN2:		ld	(hl), ' '	; vorheriges Zeichen lschen
		dec	hl
		bit	7, (hl)
		jr	z, ZLEIN3
		dec	hl		; Zeilenende belassen
ZLEIN3:		ld	(CPOS),	hl	; Bildschirmpointer
ZLEIN4:		cp	7Fh		; <ENTER>?
		jr	nz, ZLEIN1
		ld	iy, (BZ)	; Anfang Eingabezeile(n) im BWS holen
		ret

;------------------------------------------------------------------------------
; TEXTE ETC.
;------------------------------------------------------------------------------

TASTB:		db "0AKU &:",0		; Tastaturcodetabelle
KOPFT:		db "S/P-BASIC V3.3   lines:",0	; Kopftext nach Kaltstart
BFTXT:		db "bytes free"
		db  NL
RECTX:		db "RECORD"
ERRTX:		db " ERROR"
ENDZV:		db  NL			; Vorgabe END-Zeile
ENDZN:		dw 9984			; Zeilennr. 9984: 27xxh ist "LIST"-Endekennung
		db lo(TEND)		; "END"-Token
		db  NL

;------------------------------------------------------------------------------
; Berechnung der Bildschirmadr.	fr Plot/Unplot
;------------------------------------------------------------------------------

BSAPL:		call	ieval1		; Graphik-Spaltennummer in L
		ld	a, l
		cp	BZLEN+BZLEN-2	; max. Spaltennummer-1
		jp	nc, OERR	; O-ERROR (Spaltennummer-berlauf)
		ld	e, a
		call	KOMMA		; Komma	?
		call	ieval1		; Graphik-Zeilennummer in L
		ld	a, (ZZ)		; Zeilenzahl Bildschirm
		add	a, a
		dec	a		; A = max.Zeilennummer Graphik
		cp	l
		jp	c, OERR		; O-ERROR (Zeilennummer-berlauf)
		ld	a, l
		ld	h, 0
		srl	l
		ld	b, h
		ld	c, l		; HL=BC = BS-Zeilennummer
		add	hl, hl		; *** x2
		add	hl, bc		; *** x3
		add	hl, hl		; *** x6
		add	hl, bc		; *** x7
		add	hl, hl		; *** x14
		add	hl, bc		; *** x15
		add	hl, hl		; *** x30 : Mult. mit Zeichenzahl/Zeile
		ld	bc, BWS		; Beginn Bildwiederholspeicher
		add	hl, bc		; Anfang BS-Zeile
		ld	c, e
		ld	b, 0
		srl	c		; BC = Bildschirmzeichennummer
		add	hl, bc		; HL = Bildschirmadresse
		ld	(CPOS),	hl	; in Bildschirmpointer ("PRINT AT"-Ersatz)
		ld	b, 1		; B=1 bei gerader ZN und SN vorbereiten
		and	b
		jr	z, BSAP1
		sla	b
		sla	b		; B=4 bei unger. ZN & gerader SN
BSAP1:		bit	0, e
		ret	z		; wenn geradzahlige Spaltennummer
		sla	b		; B=8 bei unger. ZN & unger. SN
		ret			; B=2 bei gerader ZN & unger. SN

;------------------------------------------------------------------------------
; Beginn Interpreter
;------------------------------------------------------------------------------

BASIC:		ld	hl, (PBEG)	; BASIC-Programmbeginn
		dec	hl
		ld	a, (hl)
		cp	NL		; bereits BASIC-Programm definiert?
		jr	z, END		; ja: Warmstart

;------------------------------------------------------------------------------
; NEW
;------------------------------------------------------------------------------

NEW:		ld	c, 4
		call	BSCL1		; Bildschirm mit 4 Zeilen vorbereiten
		ei
		ld	de, KOPFT	; "S/P-BASIC V3.3   lines:"
		call	TEXT		; allg.	Textausgabe
		ld	hl, 3412h	; Anfangswert Zufallsz.
		ld	(arnd),	hl	; RND-Merkzelle
		call	ZLEIN		; Zeilenzahl eingeben
		di
		call	ZAHL		; HL = Zeilenzahl
		ld	(ZZ), hl	; Zeilenzahl Bildschirm
		call	BSCL		; BS lschen
		inc	hl
		ld	d, h
		ld	e, l		; DE = 1.Zeichen nach BWS
		inc	hl
		ld	(PBEG),	hl	; BASIC-Programmbeginn
		ld	bc, 4
		ld	(PLEN),	bc	; BASIC-Programmlnge
		inc	bc
		ld	hl, ENDZV
		ldir			; "9984	END" einspeichern
		ex	de, hl		; HL = 1.Byte nach Programm
NEW2:		ld	a, (hl)		; Test/Lschung Speicher
		cpl
		ld	(hl), a
		sub	(hl)
		jr	nz, NEW3	; Zelle nicht beschreibbar
		ld	(hl), a		; 0 einschreiben
		inc	hl
		jr	NEW2

NEW3:		dec	hl
		ld	(RAMTOP), hl	; phys.	Speicherende

;------------------------------------------------------------------------------
; MEM - Ermitteln freie Bytes
;------------------------------------------------------------------------------

MEM:		ld	de, (RAMTOP)
		ld	hl, 0FFFFh
MEM1:		inc	hl		; Zhler in HL
		ld	a, (de)
		dec	de
		and	a
		jr	z, MEM1		; solange Nullbyte
		ld	a, NL
		rst	28h		; NL
		ld	de, BFTXT	; "bytes free"
		call	TEOTZ		; "XXXX	bytes free"

;------------------------------------------------------------------------------
; BASIC-Hauptprogramm
;------------------------------------------------------------------------------

END:		ei
		ld	sp, BWS		; Beginn Stackbereich
		call	NLNZA		; NL, wenn nicht Zeilenanfang
		ld	a, '>'
		rst	28h		; Promptsymbol ausgeben
		call	ZLEIN		; Zeile	eingeben
		ld	a, (iy+0)
		cp	3Ah
		jr	c, END1		; evtl.	Zahl
		call	ZEILE		; Zeile	aufbereiten
		ld	a, ' '
		ld	(de), a		; Lschen Endekennung Bildschirm
		ld	(iy+0),	NL
		ld	(iy+3),	Lo(TEND); <NL>"END" anfgen
		ld	a, NL
		rst	28h		; NL
		ld	iy, KONV2	; Konvertierungspuffer Zeile
		jp	RUN2		; abarbeiten im Direktmodus

END1:		cp	'0'
		jp	c, SERR		; S-ERROR (keine Zahl)
		ld	(CPOS),	iy	; Anfang Eingabezeile
		call	ZEILE		; Zeile	aufbereiten
		call	EINF		; einsortieren
		jr	END

;------------------------------------------------------------------------------
; Aufbereiten der Eingabezeile
;------------------------------------------------------------------------------

ZEILE:		call	ZAHL		; Zeilennummer ermitteln
		ld	(KONV1), hl	; ablegen
		push	iy
		pop	de
		ld	iy, KONV2	; Konvertierungspuffer
		ld	bc, 3		; Lnge
ZEILE1:		ld	a, (de)
		add	a, a
		jr	nc, ZEILE2
		inc	de		; Zeilenende FFh berlesen
ZEILE2:		ld	a, (de)
		cp	7Fh		; <ENTER>?
		ret	z		; Aufbereitung fertig
		cp	"'"
		jr	nz, ZEILE5	; keine Strings
ZEILE3:		ld	(iy+0),	a
		inc	de
		inc	iy
		inc	bc
		ld	a, (de)
		add	a, a
		jr	nc, ZEILE4
		inc	de		; Zeilenende FF	berlesen
ZEILE4:		ld	a, (de)
		cp	7Fh
		ret	z		; Ende Eingabezeile
		cp	"'"
		jr	nz, ZEILE3	; bis Stringende erreicht
		jr	ZEILE11
ZEILE5:		push	bc
		ld	hl, SWTAB	; Schlsselworttabelle
		ld	bc, TEND-SWTAB	; Lnge SW-Tabelle
ZEILE6:		ld	a, (de)
		cpir
		jr	nz, ZEILE10	; Zeichen nicht gefunden
		push	de
		pop	ix		; IX = akt. Zeichenadresse
		dec	hl
		dec	hl
		bit	7, (hl)		; vorher Token?
		inc	hl
		inc	hl
		jr	z, ZEILE6	; nein:	weitersuchen, da kein Schlsselwortanfang
ZEILE7:		bit	7, (hl)		; nchstes Zeichen Token?
		jr	nz, ZEILE9	; Schlsselwort gefunden
ZEILE8:		inc	ix
		ld	a, (ix+0)	; nchstes Zeichen aus BWS
		bit	7, a
		jr	nz, ZEILE8	; Zeilenende FF	berlesen
		cpi
		jr	nz, ZEILE6	; nicht	gef. (neues Schlsselwort suchen)
		jr	ZEILE7		; weitersuchen im akt. Schlsselwort
ZEILE9:		push	ix
		pop	de		; DE = letztes Schlsselwortzeichen
		ld	a, (hl)		; Token	holen
ZEILE10:	pop	bc
ZEILE11:	ld	(iy+0),	a	; Zeichen oder Token im Konv.-puffer ablegen
		inc	de
		cp	' '
		jr	z, ZEILE1	; Leerzeichen berlesen
		inc	c		; Zeilenlnge +	1
		inc	iy		; Konv.-pufferadr. + 1
		jr	ZEILE1

;------------------------------------------------------------------------------
; Einsortieren der aufbereiteten Zeile
;------------------------------------------------------------------------------

EINF:		ld	(iy+0),	NL	; NL anhngen
		ld	(ezlen), bc	; Zeilenlnge merken
EINF1:		ld	bc, (PLEN)	; BC = BASIC-Programmlnge
		ld	hl, (PBEG)	; HL = BASIC-Programmbeginn
		jr	EINF3
EINF2:		ld	a, NL
		inc	hl
		inc	hl		; ZN berlesen
		cpir			; Zeilenende suchen
		jp	po, EINF4	; wenn Ende Programm erreicht
EINF3:		push	hl		; HL = 1. Byte ZN
		ld	e, (hl)
		inc	hl
		ld	d, (hl)		; DE = gefundene ZN
		ld	hl, (KONV1)	; HL = gesuchte ZN
		sub	a
		sbc	hl, de
		pop	hl
		jr	z, EINF5	; wenn Zeile bereits vorhanden
		jr	nc, EINF2	; wenn gesuchte ZN > gefundene ZN
		dec	hl		; sonst neue Zeile einbauen
		add	hl, bc
		ld	d, h		; DE = Ende vorheriger Zeile
		ld	e, l		; plus Restlnge = absol. Ende BASIC-Prog.
		push	bc
		ld	bc, (ezlen)	; Lnge	einzubauender Zeile
		add	hl, bc		; HL = Programmende + Lnge neue Zeile
		ex	de, hl		; DE = Ziel, HL = Quelle fr LDDR
		push	hl
		ld	hl, (RAMTOP)	; phys.	Speicherende
		sbc	hl, de		; neues Programmende grer?
		jp	m, MERR		; ja: M-ERROR (Prog. zu lang)
		pop	hl
		pop	bc		; BC wieder Restlnge
		lddr			; Lcke fr Einfgung schaffen
		inc	hl		; HL = Anfang der Lcke
EINF4:		ld	d, h
		ld	e, l		; DE=HL = Anfang Lcke=Ziel neue Zeile
		ld	hl, (PLEN)	; BASIC-Programmlnge
		ld	bc, (ezlen)	; Lnge	komprim. Eingabezeile
		add	hl, bc
		ld	(PLEN),	hl	; neue BASIC-Programmlnge
		ld	hl, KONV1	; Konvertierungspuffer Zeilennummer
		push	hl
		ldir			; neue Zeile in Lcke einbauen
		pop	de		; DE = Anfang der neuen Zeile (ZN!)
		jr	LISTUP		; Echoausgabe konv. Zeile

EINF5:		push	hl		; Lschen der Zeile im Speicher
		ld	de, (PLEN)	; BASIC-Programmlnge
		dec	de
		dec	de		; Programmlnge runterzhlen
		inc	hl
		inc	hl		; ZN berlesen
		dec	bc
		dec	bc		; Restblocklnge runterzhlen
EINF6:		ld	a, (hl)
		cp	NL		; Zeilenende suchen
		dec	bc
		dec	de		; s.o.
		inc	hl
		jr	nz, EINF6	; bis Zeilenende erreicht
		ld	(PLEN),	de	; korrigierte Programmlnge
		pop	de		; DE = 1.Byte ZN der zu streich. Zeile
		ldir			; Zeile streichen (Rest rckt nach vorn)
		ld	a, (KONV2)	; 1. Zeichen Konvertierungspuffer
		cp	NL		; nur ZN + NL ?
		jr	nz, EINF1	; nein: gestrichene Zeile neu einfgen
		ret

;------------------------------------------------------------------------------
; LIST - UP
;------------------------------------------------------------------------------

; Echoausgabe konvertierte Zeile

LISTUP:		ld	a, (de)
		ld	l, a
		inc	de
		ld	a, (de)
		ld	h, a		; HL = Zeilennummer
		inc	de

; Zeilennummer ausgeben

TEOTZ:		push	de
		ld	c, 0FFh		; Integer vorbereiten
		call	eval9		; Operand ablegen
		call	OUTNUM		; Zahl ausgeben
		pop	de

; allg.	Textausgabe

TEXT:		ld	a, (de)
		and	a
		ret	z		; wenn Textendekennzeichen 0
		jp	p, TEXT3	; kein Token
		ld	hl, SWTAB	; Schlsselworttabelle
		ld	bc, TEND-SWTAB	; Lnge Schlsselworttabelle
		cpir			; Token suchen
		ld	b, 0FFh
		dec	hl
TEXT1:		dec	hl
		inc	b		; B zhlt Lnge Schlsselwort
		bit	7, (hl)
		jr	z, TEXT1	; bis rckwrts vorheriger Token gefunden
TEXT2:		inc	hl
		ld	a, (hl)
		or	20h		; Schlsselwort in Kleinbuchstaben umwandeln
		rst	28h
		djnz	TEXT2
		ld	a, ' '          ; Leerzeichen nach Schlsselwort
TEXT3:		rst	28h
		cp	NL
		inc	de
		jr	nz, TEXT	; bis Zeilenende
		ret

; eigentliche LIST-Routine

LIST:		ld	de, (PBEG)	; BASIC-Programmbeginn
LIST1:		ld	a, (ZZ)		; Zeilenzahl Bildschirm
		rra			; Cy war 0
		dec	a
		ld	b, a		; B = List-Zeilenzahl: INT(ZZ)/2 - 1
LIST2:		inc	de
		ld	a, (de)
		cp	hi(9984)	; ZN 9984 (=2700h) erreicht?
		jp	z, MEM		; Ermitteln und Ausgeben freie Bytes
		dec	de
		push	bc
		call	LISTUP		; Echoausgabe konvertierte Zeile
		pop	bc
		djnz	LIST2
		call	ZEIN		; beliebiges Zeichen eingeben
		jr	LIST1

;------------------------------------------------------------------------------
; RUN
;------------------------------------------------------------------------------

RUN:		call	BSCL		; BS lschen
		ld	de, (PBEG)	; BASIC-Programmbeginn
		push	de
		ld	hl, (PLEN)	; BASIC-Programmlnge
		add	hl, de
		ld	(hl), 0		; 1. Byte nach Programm lschen
		ex	de, hl
		ld	hl, (RAMTOP)	; phys.	Speicherende
		and	a
		sbc	hl, de
		ld	b, h
		ld	c, l		; BC = Lnge Arbeitsspeicher
		ld	h, d
		ld	l, e		; HL = 1. Byte Arbeitsspeicher
		inc	de
		ldir			; Speicher lschen
		pop	iy		; BASIC-Programmbeginn
RUN1:		inc	iy		; neue Zeile abarbeiten
		inc	iy
RUN2:		ld	a, (iy+0)	; nchste Anweisung abarbeiten
		and	a
		jp	p, LET		; LET, wenn kein Token
		cp	Lo(TEND)
		jp	m, FERR		; F-ERROR (keine Anw.)
		inc	iy
		ld	h, Hi(TEND)	; H-Adresse Tokentabelle
		ld	l, a
		ld	a, (hl)
		inc	l
		ld	h, (hl)
		ld	l, a		; HL = Routinenadresse aus Tokentabelle
		jp	(hl)		; Sprung zur Routine

;------------------------------------------------------------------------------
; RANDOM
;------------------------------------------------------------------------------

RANDO:		ld	a, r
		ld	(arnd),	a	; RND-Merkzelle
		jr	RETRN1

;------------------------------------------------------------------------------
; GOTO
;------------------------------------------------------------------------------

GOTO:		call	ieval1		; Integer-Ausdruck berechnen
GOTO1:		ld	hl, (PBEG)	; BASIC-Programmbeginn
		ld	bc, (PLEN)	; BASIC-Programmlnge
GOTO2:		ld	a, (alh2)	; L(gesuchte Zeilennummer)
		cpi
		jr	nz, GOTO3	; nicht gefunden
		ld	a, (alh2+1)	; H(gesuchte Zeilennummer)
		cpi
		jr	nz, GOTO3	; nicht gefunden
		push	hl
		pop	iy		; 1. Zeichen der Zeile
		jr	RUN2		; (1.) Anw. abarbeiten
GOTO3:		ld	a, NL
		cpir			; Zeilenende suchen
		jp	pe, GOTO2	; Programmende noch nicht erreicht
		ld	a, 'L'
		jr	JPERR		; L-ERROR (Zeile nicht gefunden)

;------------------------------------------------------------------------------
; IF
;------------------------------------------------------------------------------

IF:		call	ieval1		; Integer-Ausdruck berechnen
		ld	a, (iy+0)
		cp	Lo(TTHEN)	; "THEN" ?
		jr	nz, IF1
		inc	iy		; "THEN" berlesen
IF1:		ld	a, h
		or	l
		jr	nz, RUN2	; Bedingung erfllt: nchste Anw. abarb.

;------------------------------------------------------------------------------
; REM / DATA  -  bis Ende der Programm-ZEILE berlesen
;------------------------------------------------------------------------------

REM:		ld	b, 76		; max. Zeilenlnge
REM1:		ld	a, (iy+0)
		inc	iy
		cp	NL		; Zeilenende?
		jr	z, RUN1		; ja: nchste Zeile abarbeiten
		djnz	REM1		; entsprechend B weitersuchen
		jr	SERR		; kein Zeilenende gefunden

;------------------------------------------------------------------------------
; nchste Anweisung im BASIC-Programm suchen (":" oder Zeilenende)
;------------------------------------------------------------------------------

NANWS:		ld	b, 76		; max. Zeilenlnge
		jr	RETRN2

;------------------------------------------------------------------------------
; RETURN
;------------------------------------------------------------------------------

RETRN:		pop	iy
RETRN1:		ld	b, 1
RETRN2:		ld	a, (iy+0)
		cp	':'		; Anweisungstrenner ":"?
		inc	iy
		jr	z, RUN2		; ja: nchste Anweisung abarbeiten
		cp	NL		; Zeilenende?
		jr	z, RUN1		; ja: nchste Zeile abarbeiten
		djnz	RETRN2		; entsprechend B weitersuchen
SERR:		ld	a, 'S'          ; S-ERROR (Anweisung bzw. Zeile zu lang)
JPERR:		jp	ERROR

;------------------------------------------------------------------------------
; GOSUB
;------------------------------------------------------------------------------

GOSUB:		call	ieval1		; Integer-Ausdruck berechnen
		push	iy		; Rckkehradr. merken
		jr	GOTO1

;------------------------------------------------------------------------------
; INPUT
;------------------------------------------------------------------------------

INPUT:		call	FORMAT		; Formatsteuerung
		jr	z, RETRN1	; wenn Anweisungs- oder Zeilenende
		call	PRNZK		; Strings?
		jr	z, INPUT	; ja
		call	VARADR		; Var.-Adr. ermitteln
		push	ix		; Var.-Adr. merken
		push	iy		; BASIC-Adr. merken
		call	ZLEIN		; Zeile	eingeben
		ld	a, d
		cp	'$'
		jr	nz, INPUT1	; keine	Strings
		call	STRING1		; Stringeingabe
		jr	INPUT2
INPUT1:		ld	hl, 0
		ld	c, 0FFh
		ld	a, (iy+0)
		cp	7Fh		; sofort Endekennung?
		call	nz, eval1	; nein: Wert berechnen
INPUT2:		ld	(iy+0),	' '     ; Lschen Endezeichen
		pop	iy		; BASIC-Adr. holen
		pop	ix		; Var.-Adr. holen
		call	LETUP2		; Wert ablegen
		jr	INPUT

;------------------------------------------------------------------------------
; PRINT
;------------------------------------------------------------------------------

PRINT:		call	FORMAT		; Formatsteuerung
		jr	z, PRINT1	; wenn Anweisungs- oder Zeilenende
		call	PRNZK		; Strings oder Hex.?
		jr	z, PRINT	; Ja
		call	eval1		; Wert berechnen
		call	OUTVAL		; Wert ausgeben
		jr	PRINT
PRINT1:		ld	a, NL
		rst	28h		; NL ausgeben
		jr	RETRN1

;------------------------------------------------------------------------------
; CLS
;------------------------------------------------------------------------------

CLS:		call	BSCL		; BS lschen
		jr	RETRN1

;------------------------------------------------------------------------------
; LET
;------------------------------------------------------------------------------

LET:		call	LETUP
		ld	a, (iy-1)
		cp	'+'             ; Stringverkettung?
		call	z, LETUP1	; Stringaddition ausfhren
		call	KOMMA		; Komma?
		jr	LET

;------------------------------------------------------------------------------
; DIM
;------------------------------------------------------------------------------

DIM:		call	VARADR		; Var.-Adr.
		ld	a, '('
		cp	(iy+0)
		jp	nz, ERROR	; (-ERROR (ffnende Klammer fehlt)
		call	ieval		; 1. Integerausdruck
		set	7, (ix+0)	; Kennung eindimensionales Feld
		ld	d, h
		ld	e, l		; DE=HL = 1.Grenzindex
		ld	a, (iy+0)
		cp	','
		jr	nz, DIM1	; kein 2. Grenzindex
		ex	de, hl
		call	ieval		; 2. Grenzindex berechnen
		ex	de, hl		; DE = 2. und HL = 1.Grenzindex
		ld	b, d
		ld	c, e		; BC = 2. Grenzindex
		call	MHLBC		; HL = HL * BC : Elementezahl
		set	7, (ix+1)	; Kennung: zweidimensional
DIM1:		call	BREXI2		; Klammerabfrage
		ld	a, (ix+1)
		and	7Fh
		cp	'$'             ; Strings?
		jr	z, DIM2
		add	hl, hl
		add	hl, hl		; bei neuem Feld: Lnge = Elementezahl*4
DIM2:		ld	(ix+4),	e	; 2. Grenzindex
		ld	(ix+5),	d	; (bei eindim. 1. Grenzindex)
		ld	(ix+2),	l
		ld	(ix+3),	h	; (Elementezahl =) Lnge in Bytes
		ex	de, hl
		add	ix, de		; IX = nchste Adr. im Speicher
		call	MEMEND		; Speicherende?
		call	KOMMA		; Komma?
		jr	DIM

;------------------------------------------------------------------------------
; FOR
;------------------------------------------------------------------------------

FOR:		call	LETUP		; LET-UP (Erstzuweisung Laufvariable)
		push	ix
		push	iy
JPNANW:		jp	NANWS		; nchste Anweisung suchen

;------------------------------------------------------------------------------
; NEXT
;------------------------------------------------------------------------------

NEXT:		ld	(ezlen), iy	; Adr. fr Weiterarbeit nach NEXT merken
		pop	iy
		pop	ix		; Variablen- und Programmadresse holen
		push	ix		; und wieder retten (aus zugehr. "FOR")
		push	iy
		ld	a, (iy+0)
		cp	Lo(TTO)		; TO?
		jp	nz, SERR	; S-ERROR (wenn "TO" fehlt)
		call	eval		; Endwert berechnen
		call	GETVAR		; Var.-Wert holen
		call	COMP4		; Endwert erreicht?
		jr	nz, NEXT1	; nein (Var.-Wert <> Endwert)
		pop	de
		pop	de		; Stack bereinigen
		ld	iy, (ezlen)
		jr	JPNANW		; Fortsetzen nach NEXT

NEXT1:		push	bc		; Var.-Wert merken
		push	hl
		ld	hl, 1		; STEP 1 vorbereiten
		ld	c, 0FFh
		ld	a, (iy+0)
		cp	Lo(TSTEP)	; STEP?
		call	z, eval		; Schrittweite berechnen
		call	eval9		; in Arithmetikpuffer (2.Operand)
		pop	hl		; Variablenwert holen
		pop	bc
		call	eval8		; in Arithmetikpuffer (1.Operand)
		push	ix
		call	AR_ADD		; Addition der Schrittweite
		pop	ix
		call	PUTVAR		; neuen	Var.-Wert ablegen
		jr	JPRET		; nchste Anweisung (Schleifenkrper)

;------------------------------------------------------------------------------
; POKE
;------------------------------------------------------------------------------

POKE:		call	ieval1		; Adr. berechnen
POKE1:		call	KOMMA		; Komma	?
		push	hl
		call	ieval1		; Integer-Ausdruck berechnen
		ld	c, l
		pop	hl
		ld	(hl), c		; Wert ablegen
		inc	hl
		jr	POKE1

;------------------------------------------------------------------------------
; OUT
;------------------------------------------------------------------------------

OUT:		call	ieval1		; Adresse berechnen
		ld	c, l
OUT1:		call	KOMMA		; Komma	?
		call	ieval1		; Wert berechnen
		out	(c), l
		jr	OUT1

;------------------------------------------------------------------------------
; PLOT
;------------------------------------------------------------------------------

PLOT:		call	BSAPL		; HL = Adr.BWS, B = Graphikcode (1,2,4,8)
		ld	a, (hl)
		cp	' '
		jr	c, PLOT1	; wenn Graphikzeichen im BWS
		xor	a		; sonst A=0: leeres Graphikfeld
PLOT1:		or	b		; zustzl. Pseudgraphikzeichen einbauen
		jr	UNPLT1

;------------------------------------------------------------------------------
; UNPLOT
;------------------------------------------------------------------------------

UNPLT:		call	BSAPL		; HL = Adr.BWS, B = Graphikcode (1,2,4,8)
		ld	a, (hl)
		cpl
		or	b		; Zeichen weglschen
		cpl
UNPLT1:		ld	(hl), a
JPRET:		jp	RETRN1

;------------------------------------------------------------------------------
; RESTO
;------------------------------------------------------------------------------

RESTO:		ld	hl, (PBEG)	; BASIC-Programmbeginn
		ld	bc, (PLEN)	; BASIC-Programmlnge
RESTO1:		ld	a, Lo(TDATA) 	; "DATA" ?
		cpir
RESTO2:		ld	a, 'D'
		jp	nz, ERROR	; D-ERROR: kein "DATA" im Programm
		ld	d, h
		ld	e, l		; DE = Adr. fr DATA
		dec	de
		dec	de
		dec	de
		dec	de
		ld	a, (de)
		cp	NL
		jr	nz, RESTO1	; weitersuchen wenn nicht am Zeilenanfang
		ld	(READPT), de	; READ-Pointer auf 1.DATA (konkret NL vorher. Zeile)
		jr	JPRET

;------------------------------------------------------------------------------
; READ
;------------------------------------------------------------------------------

READ:		call	VARADR		; Variablenadresse ermitteln
		push	iy
		ld	iy, (READPT)	; READ-Pointer
		ld	a, ','
		cp	(iy+0)
		jr	z, READ1	; wenn Komma
		inc	iy
		inc	iy
		inc	iy		; Endekennz. und ZN berlesen
		ld	a, Lo(TDATA) 	; "DATA" ?
		cp	(iy+0)
		jr	nz, RESTO2	; D-ERROR: kein "DATA" am Zeilenanfang
READ1:		call	eval		; Wert berechnen
		ld	(READPT), iy	; READ-Pointer aktualisieren
		pop	iy
		call	LETUP2		; Wert ablegen
		call	KOMMA		; Komma	?
		jr	READ

;------------------------------------------------------------------------------
; CALL
;------------------------------------------------------------------------------

CALL:		call	ieval1		; Zieladresse berechnen
		push	iy		; RETURN-Adr. merken
		ld	de, RETRN
		push	de		; Wiedereinsprung in RETURN nach RET im UP
		jp	(hl)		; eigentliches UP anspringen

;------------------------------------------------------------------------------
; Formatsteuerung
;------------------------------------------------------------------------------

FORMAT:		ei
		ld	a, (iy+0)
		cp	';'
		jr	z, FORMAT5	; bei Semikolon nix machen
		cp	','
		jr	z, FORMAT1	; zur nchsten Tabulatorpos. vorsetzen
		cp	':'
		ret	z		; wenn Anweisungsende: Z=1
		cp	NL
		ret	z		; wenn Zeilenende: Z=1
		ld	a, (iy-1)
		and	a
		ret	m		; wenn vorher Token (Anfang der Anweisung)
		jp	SERR		; S-ERROR wenn vorher was anderes steht
FORMAT1:	ld	b, 0
		ld	hl, (CPOS)	; Bildschirmpointer
FORMAT2:	inc	hl
		inc	b
		bit	7, (hl)
		jr	z, FORMAT2	; zum Zeilenende vorsetzen
		ld	a, b		; A = Restzeichenzahl der Zeile
FORMAT3:	sub	7		; Spaltenbreite = 7
		jr	z, FORMAT4
		jr	nc, FORMAT3
FORMAT4:	add	a, 7		; Korrektur
		ld	b, a		; vorzusetzende LZ
		call	LZW		; Tabulator ausfhren
FORMAT5:	inc	iy
		ld	a, (iy+0)	; nachfolg. Zeichen analysieren
		cp	':'		; wenn Anweisungsende
		jr	z, KOMMA1
		cp	','		; wenn noch eine Tabulatorpos. vor
		jr	z, FORMAT1
		cp	NL
		ret	nz		; wenn Ausdruck folgt
		jr	KOMMA1		; wenn Zeilenende

;------------------------------------------------------------------------------
; Komma	abfragen
;------------------------------------------------------------------------------

KOMMA:		ld	a, (iy+0)
		cp	','
		jr	z, KOMMA2
KOMMA1:		pop	de		; Stack korrigieren (Rckkehradresse vergessen)
		jp	RETRN1		; nchste Anw. abarbeiten
KOMMA2:		inc	iy		; Komma berlesen
		ret			; weiter in aktueller Anweisung

;------------------------------------------------------------------------------
; Ausgabe von Werten
;------------------------------------------------------------------------------

OUTVAL:		ld	a, c
		cp	0FEh
		jr	nz, OUTNUM	; keine Strings
OUTSTR:		ld	a, (hl)
		and	a
		ret	z		; wenn Stringende
		rst	28h
		inc	hl
		djnz	OUTSTR		; entsprechend Lnge in B
		ret

;------------------------------------------------------------------------------
; Zahl ausgeben
;------------------------------------------------------------------------------

OUTNUM:		ld	hl, acb2	; Ausdruckwert CB
		call	TOBCD1		; 2.Operand (Ergebnis) Integer ---> BCD
		ld	hl, asign2	; Vorzeichen 2.Operand
		bit	7, (hl)
		jr	z, OUTNUM1	; wenn positiv
		ld	a, '-'
		rst	28h		; Minus ausgeben
OUTNUM1:	ld	a, (hl)
		and	7		; Stelligkeit nach Komma
		sub	8
		ld	b, a		; B = Stellen nach Komma
		ld	c, 0
		ld	hl, acb2	; Ausdruckwert CB
		rld
		call	outz		; Ausgabe-Hilfsprogramm
		call	outz0		; Ausgabe-Hilfsprogramm
		call	outz0		; Ausgabe-Hilfsprogramm
OUTNUM2:	call	outz0		; Ausgabe-Hilfsprogramm
		bit	7, c		; 0 wenn Wert = 0
		call	z, ZAUS		; Zeichenausgabe
		ld	a, ' '
		rst	28h		; Leerzeichen danach
		ret

outz0:		inc	hl
		call	outz
;
outz:		inc	b
		jr	nz, outz1	; Komma noch nicht auszugeben
		ld	a, '.'
		rst	28h		; Dezimalpunkt ausgeben
		dec	c
outz1:		sub	a
		rld
		and	a
		jr	z, outz2	; wenn Tetrade = 0
		dec	c		; Vornullunterdrckung aus
outz2:		or	30h
		cp	3Ah
		jr	c, outz3	; Zahl
		add	a, 7		; Korrektur Hex. ---> Binr
outz3:		bit	7, c
		ret	z		; wenn Vornullunterdrckung ein
		rst	28h		; Stelle ausgeben
		ret

;------------------------------------------------------------------------------
; Ausgabe Strings, Hex. und SPC-Funktion fr PRINT
;------------------------------------------------------------------------------

; Stringausgabe
PRNZK:		ld	a, (iy+0)
		cp	"'"
		jr	nz, PRNZK2	; keine Strings
		inc	iy
PRNZK1:		ld	a, (iy+0)
		cp	NL
		ret	z		; Zeilenende
		inc	iy
		cp	"'"
		ret	z		; Stringende
		rst	28h		; Zeichen ausgeben
		jr	PRNZK1

PRNZK2:		cp	'%'
		jr	nz, PRNZK3	; keine Hex.-Ausgabe
		call	ieval		; Integer-Ausdruck berechnen
		ld	hl, alh2	; Ausdruckwert LH
		ld	bc, 0		; kein Komma, Vornullunterdrckung
		call	outz0		; hherwertiges Byte hex.
		dec	hl
		dec	hl
		call	OUTNUM2		; niederwertiges Byte hex.
		xor	a
		ret

PRNZK3:		cp	'&'
		ret	nz		; Fkt. nicht ausgefhrt
		call	ieval		; Integer-Ausdruck berechnen
		ld	b, l		; B = SPC-Zahl

;------------------------------------------------------------------------------
; B Leerzeichen	ausgeben (Tab)
;------------------------------------------------------------------------------

LZW:		ld	a, ' '
LZW1:		rst	28h
		djnz	LZW1
		ret

;------------------------------------------------------------------------------
; LET-UP
;------------------------------------------------------------------------------

LETUP:		call	VARADR		; Var.-Adr.
		ld	a, (iy+0)
		cp	'='
		jp	nz, SERR	; S-ERROR (kein Gleichheitszeichen bei LET)
		inc	iy
LETUP1:		call	eval1		; Wert berechnen

; Variablenwert ablegen

LETUP2:		ld	a, d
		cp	'$'
		jr	nz, PUTVAR	; Zielvariable kein String
		ld	a, c
		cp	0FEh		; Quellvariable String?
STERR:		ld	a, '$'
		jp	nz, ERROR	; $-ERROR bei Type Mismatch
LETUP3:		ld	a, (hl)		; Quellzeichen holen
		ld	(ix+6),	a	; ... und in Zielvariable ablegen
		and	a
		ret	z		; bei Quellstring-Ende = 0
		inc	hl
		inc	ix
		dec	e		; E = maximale bzw. Vorgabelnge Zielstring
		ret	z		; wenn Vorgabelnge erreicht
		djnz	LETUP3		; forts. bis max. Lnge Quelle entspr. B erreicht
		ld	(ix+6),	0	; Endekennung Zielstring
		ret
PUTVAR:		ld	(ix+2),	c	; numerischen Variablenwert ablegen
		ld	(ix+3),	b
		ld	(ix+4),	l
		ld	(ix+5),	h
		ret

;------------------------------------------------------------------------------
; Variablenadresse ermitteln
;------------------------------------------------------------------------------

VARADR:		ld	ix, (PBEG)	; BASIC-Programmbeginn
		ld	bc, (PLEN)	; BASIC-Programmlnge
		add	ix, bc		; 1. Byte nach Programm (Beginn Arbeitsspeicher)
		ld	a, (iy+0)
		cp	40h
		ld	b, a		; B = 1. Zeichen
		ld	a, 'U'
		jp	m, ERROR	; U-ERROR (kein Buchstabe)
		ld	c, 0		; vorbereiten kein zweites Zeichen
		ld	d, c
		inc	iy
		ld	a, (iy+0)
		and	a
		jp	m, VARAD2	; Token ---> kein zweites Zeichen
		cp	'$'
		jr	z, VARAD1	; 2. Zeichen $
		cp	'0'
		jr	c, VARAD2	; keine Zahl
		cp	40h
		jr	nc, VARAD1	; 2. Zeichen Buchstabe
		cp	3Ah
		jr	nc, VARAD2	; 2. Zeichen keine Zahl
VARAD1:		ld	c, a		; C = 2. Zeichen
		inc	iy
VARAD2:		ld	a, b
		sub	(ix+0)
		and	7Fh
		jp	nz, VARAD7	; 1. Zeichen nicht gefunden
		ld	a, (ix+1)
		and	7Fh
		cp	c
		jp	nz, VARAD7	; 2. Zeichen nicht gefunden
		ld	e, (ix+4)	; vorb. Grenzindex $ als Zeichenzahl
		bit	7, (ix+0)
		ret	z		; kein (eindim.) Feld bzw. String
		ld	d, a		; D = 2. Zeichen
		ld	a, '('
		cp	(iy+0)
		ret	nz		; keine Feldvar. oder kompletter String lt. Lnge in B
		call	ieval		; 1. Index berechnen
		dec	hl		; 1. Index korrigieren
		bit	7, h
		jr	nz, VARAD3	; O-ERROR (Index <= 0)
		ld	b, (ix+5)
		ld	c, e		; BC = (1./2.) Grenzindex ("Zeilenlnge")
		bit	7, (ix+1)
		jr	z, VARAD4	; kein zweidim. Feld
		call	MHLBC		; HL=BC := HL*BC = Zeilenanfangsadresse
		ld	a, (iy+0)
		cp	','
		jr	nz, VARAD4	; kein Komma: 2. Index fehlt (als 1 anges. bes. bei Strings)
		call	ieval		; 2. Index berechnen
		dec	hl		; 2. Index korrigieren
		bit	7, h
VARAD3:		jp	nz, OERR	; O-ERROR (Index <= 0)
		add	hl, bc		; HL = Adr. im Feld = 1.Index * 2.Grenzindex + 2.Index
VARAD4:		ld	a, '$'
		cp	d
		jr	z, VARAD5	; wenn Stringfeld
		inc	hl
		add	hl, hl
		add	hl, hl		; *4 fr neue Var + 4 fr berlesen Kennung
		jr	VARAD6
VARAD5:		ld	a, (iy+0)
		cp	','
		jr	nz, VARAD6	; keine Zeichenspezifikation bei Strings
		push	hl
		call	ieval		; 3. Index berechnen
		ld	e, l		; E = Zeichenzahl String
		pop	hl
VARAD6:		call	BREXI2		; Klammerabfrage ')'
		ld	b, h
		ld	c, l		; BC = Adr. im Feld
		ld	l, (ix+2)
		ld	h, (ix+3)	; HL = Feldlnge in Bytes
		sub	a
		sbc	hl, bc
		jp	m, OERR		; O-ERROR (Indexberlauf)
		add	ix, bc		; Var.adr. + Relativadr.Feld (-2 bei num.; -6 bei $)
		ret

; Hilfsprogramm BC=HL := HL*BC

MHLBC:		push	de
		ex	de, hl
		ld	hl, 0
		ld	a, 17		; Zykluszahl
MHLBC1:		rr	b
		rr	c
		dec	a
		jr	z, MHLBC3
		jr	nc, MHLBC2
		add	hl, de
MHLBC2:		rr	h
		rr	l
		jr	MHLBC1
MHLBC3:		pop	de
		ld	h, b
		ld	l, c
		ret

;
VARAD7:		ld	a, (ix+0)
		and	a
		jr	z, VARAD8	; Ende Speicher erreicht
		inc	ix
		inc	ix
		inc	ix		; berlesen Name und Wert (bei num.)
		inc	ix		; bzw. Kenndaten (bei $ oder Feldern)
		inc	ix
		inc	ix
		jp	p, VARAD2	; nchste Var. KEIN Feld bzw. KEIN $
		push	bc
		ld	c, (ix-4)
		ld	b, (ix-3)	; BC = Feldlnge in Bytes
		add	ix, bc		; IX auf Name nchste Variable
		pop	bc
		jp	VARAD2		; weitersuchen
;
VARAD8:		ld	(ix+0),	b	; neue Variable anlegen
		ld	(ix+1),	c

;------------------------------------------------------------------------------
; Speicherende?
;------------------------------------------------------------------------------

MEMEND:		push	ix
		pop	bc
		ld	hl, (RAMTOP)	; phys.	Speicherende
		sbc	hl, bc
		ld	bc, 8		; 8 Byte Reserve lassen
		sbc	hl, bc
		ret	p		; noch mehr als 8 Byte frei
MERR:		ld	a, 'M'
JPERRM:		jp	ERROR		; M-ERROR

;------------------------------------------------------------------------------
; Integer-Ausdrcke berechnen
;------------------------------------------------------------------------------

ieval:		inc	iy
ieval1:		push	bc
		call	eval1		; Ausdruck berechnen
		inc	c
		ld	a, 'I'
		jr	nz, JPERRM	; I-ERROR wenn nicht integer
		pop	bc
		ret

;------------------------------------------------------------------------------
; allg.	Ausdrcke berechnen
;------------------------------------------------------------------------------

eval:		inc	iy
eval1:		push	ix
		push	de
		ld	e, 0		; Verschachtelungszhler
		call	OPBER		; 1. Operand berechnen
eval2:		ld	a, d
		and	a
		jp	m, eval7	; wenn Ausdr.-Ende (C3)
eval3:		push	bc
		push	hl		; Wert merken
		ld	a, d
		push	af		; Prioritt merken
		call	OPBER		; weiteren Operand berechnen
eval4:		pop	af		; vorher. Prioritt
		push	af
		cp	d
		jp	p, eval5	; wenn gleiche oder niedrigere
		inc	e		; Prioritt folgt (bzw.	Ende C3)
		jr	eval3
eval5:		push	de		; berechnen Block hherer Prioritt
		call	eval9		; 2. Operand ablegen
		pop	de
		ld	b, 0
		ld	c, a
		ld	ix, ARTAB	; Arithmetik-Sprungverzweiger
		add	ix, bc		; Zieladr. Arithmetik
		pop	af
		pop	hl
		pop	bc		; 1. Operand holen
		push	de
		call	eval8		; 1. Operand ablegen
		ld	de, eval6
		push	de		; Rckkehradr.
		jp	(ix)		; Sprung zur Arithmetik-Routine

eval6:		pop	de
		ld	a, e
		and	a
		jr	z, eval2	; Verschachtelung abgearbeitet
		dec	e
		jr	eval4

eval7:		call	eval9		; Ende des Ausdrucks
		pop	de
		pop	ix
		ret

eval8:		ld	de, areg1	; Arithmetikpuffer
		jr	eval10		; 1. Operand ablegen
eval9:		ld	de, areg2	; 2. Operand (Ergebnis) ablegen
eval10:		ex	de, hl
		push	af
		sub	a		; A=0
		ld	(hl), a
		inc	hl
		ld	(hl), a
		inc	hl
		ld	(hl), a
		inc	hl
		ld	(hl), a
		inc	hl
		ld	a, c		; Arithmetikpuffer gelscht
		cp	0FFh
		jr	z, eval11	; bei Integer
		and	0Fh		; "Millionen", hchste Tetrade
eval11:		ld	(hl), a		; "C" ablegen
		inc	hl
		ld	(hl), b		; "B" ablegen
		inc	hl
		ld	(hl), e		; "L" ablegen
		inc	hl
		ld	(hl), d		; "H" ablegen
		inc	hl
		ld	(hl), c
		ld	a, c
		rld
		and	87h
		ld	(hl), a		; Vorz. und Stellenzahl nach Komma ablegen
		pop	af
		ex	de, hl
		ret

;------------------------------------------------------------------------------
; Operand berechnen
;------------------------------------------------------------------------------

evop:		ld	a, (iy+0)
		cp	'('
		jp	z, BREXI1	; Klammerausdruck berechnen
		cp	'-'
		jp	nz, STRING	; bei "-" Integer=0 ansetzen; Minus selbst
					; wird ber Arithmetik ausgefhrt (0 minus Wert)
		ld	l, 0
		jr	PEEK2

;------------------------------------------------------------------------------
; CHR$
;------------------------------------------------------------------------------

CHRS:		and	a
		jp	p, CHRS1	; Zahl oder Variable
		cp	Lo(TCHR) 	; CHR$ ?
		jr	nz, INKEY
		call	BREXI		; Klammerausdruck berechnen
		ld	a, l
		ld	hl, CHRMZ	; Merkzelle fr	CHR$
		jr	INKEY1

;------------------------------------------------------------------------------
; INKEY
;------------------------------------------------------------------------------

INKEY:		cp	Lo(TINKEY) 	; INKEY?
		jr	nz, INT
		inc	iy
		call	TSTS		; Tastaturstatus
		ld	hl, TACOD	; Tastencode bei INKEY$
INKEY1:		ld	(hl), a
		ld	bc, 01feh	; B=1 (Lnge), C = Stringkennung
		ret

;------------------------------------------------------------------------------
; ASC
;------------------------------------------------------------------------------

ASC:		cp	Lo(TASC) 	; ASC ?
		jr	nz, PEEK
		call	BREXI1		; Klammerausdruck berechnen
		ld	a, c
		cp	0FEh
		jp	nz, STERR	; wenn kein String
		jr	PEEK1

;------------------------------------------------------------------------------
; PEEK
;------------------------------------------------------------------------------

PEEK:		cp	Lo(TPEEK) 	; PEEK ?
		jr	nz, IN
		call	BREXI		; Klammerausdruck berechnen
PEEK1:		ld	l, (hl)
PEEK2:		ld	h, 0
PEEK3:		ld	c, 0FFh		; C=FFh (Integer), HL=Wert
		ret

;------------------------------------------------------------------------------
; IN
;------------------------------------------------------------------------------

IN:		cp	Lo(TIN) 	; IN ?
		jr	nz, RND
		call	BREXI		; Klammerausdruck berechnen
		ld	c, l
		in	l, (c)
		jr	PEEK2

;------------------------------------------------------------------------------
; RND
;------------------------------------------------------------------------------

RND:		cp	Lo(TRND) 	; RND ?
		jr	nz, USR
		call	BREXI		; max. Wert berechnen
		ld	a, l
		or	h
		jr	z, RND4		; RND(0) stets 1
		push	hl
		ld	hl, (arnd)	; RND-Merkzelle
		ld	b, 7		; Pseudozufallszahlenschleife
RND1:		ld	a, 0B4h
		and	h
		add	hl, hl
		jp	po, RND2
		inc	l
RND2:		djnz	RND1		; bis alle 7 Zyklen durch
		ld	(arnd),	hl	; RND-Merkzelle
		pop	bc		; Vorgabewert x (integer!) fr RND(x)
		and	a
RND3:		sbc	hl, bc		; entsprechend oft von RND-MZ abziehen, bis negativ
		jr	nc, RND3	; hoher Zeitbedarf!! Tip: groes x vorgeben und divid.
		add	hl, bc		; Korrektur auf positiv
RND4:		inc	hl		; +1: RND(x) = 1 bis x
		jr	PEEK3

;------------------------------------------------------------------------------
; INT
;------------------------------------------------------------------------------

INT:		cp	Lo(TINT) 	; INT?
		jr	nz, ASC
		call	BREXI1		; Klammerausdruck berechnen
		ld	a, c
		inc	a
		ret	z		; bereits Integer
		ld	a, (asign2)	; Vorzeichen
		and	7
		ld	c, a		; C = Stellenzahl nach Komma
INT1:		call	DIV10		; Kommastellen wegschieben (Div. durch 10)
		dec	c
		jr	nz, INT1
		ld	a, (asign2)	; Vorzeichen
		and	a
		jp	p, AR_MULT12	; wenn posit. Wert
		ld	hl, 0FFFFh
		ld	c, h		; "-1"
		call	eval8
		jp	AR_ADD		; bei negativ: Wert - 1

;------------------------------------------------------------------------------
; STRING
;------------------------------------------------------------------------------

STRING:		cp	"'"
		jp	nz, CHRS
		inc	iy
STRING1:	push	iy
		pop	hl
		ld	bc, 0FEh
STRING2:	ld	a, (iy+0)
		cp	7Fh
		ret	z		; wenn Eingabeende
		cp	NL
		ret	z		; wenn Zeilenende
		inc	b		; B wird Stringlnge
		inc	iy
		cp	"'"
		jr	nz, STRING2
		dec	b
		ret

;------------------------------------------------------------------------------
; USR
;------------------------------------------------------------------------------

USR:		cp	Lo(TUSR) 	; USR ?
		jr	nz, LEN
		call	ieval		; Int.-Ausdr. ber.
		ex	de, hl		; DE = Zieladr.
		ld	a, (iy+0)
		cp	','
		call	z, eval		; Parameter in CBLH
		call	BREXI2		; Klammerabfrage
		push	iy
		ld	iy, USRRET
		push	iy
		push	de
		pop	iy		; IY = Zieladr.
		jp	(iy)
;
USRRET:		pop	iy
		ret

;------------------------------------------------------------------------------
; LEN
;------------------------------------------------------------------------------

LEN:		cp	Lo(TLEN) 	; LEN ?
FERR:		ld	a, 'F'          ; F-ERROR (keine Anw.)
LEN1:		jp	nz, ERROR
		inc	iy
		call	WERT		; Var.-Wert holen
		inc	c
		inc	c
		jp	nz, STERR	; $-ERROR
		dec	c
		ld	l, b
		ld	h, 0
		jr	BREXI2		; Klammerabfrage ')'

;------------------------------------------------------------------------------
; Klammerausdruck berechnen
;------------------------------------------------------------------------------

BREXI:		call	ieval		; Integer-Klammerausdr. berechnen
		jr	BREXI2		; Klammerabfrage ')'
;
BREXI1:		call	eval		; Klammerausdr.	ber.

; Klammerabfrage ')'

BREXI2:		ld	a, ')'
		cp	(iy+0)
		jr	nz, LEN1
		inc	iy
		ret

; Einsprung von CHR$
CHRS1:		cp	40h
		jp	m, ZAHL		; Zahlenwert

;------------------------------------------------------------------------------
; Var.-Wert holen
;------------------------------------------------------------------------------

WERT:		call	VARADR		; Variablenadresse ermitteln
		ld	a, d
		cp	'$'
		jr	nz, GETVAR	; keine Stings: Variablenwert holen
		push	ix
		pop	hl		; HL = Variablenadr. - 6
		ld	bc, 6
		add	hl, bc		; HL = echte Abfangsadr. String
		ld	c, 0FEh
WERT1:		ld	a, (ix+6)
		and	a
		ret	z		; wenn Stringende
		inc	b		; B = Stringlnge
		inc	ix
		ld	a, b
		cp	e
		jr	nz, WERT1	; fortsetzen bis max. Lnge erreicht
		ret

GETVAR:		ld	b, (ix+3)	; numerischen Variablenwert holen
		ld	c, (ix+2)
		ld	h, (ix+5)
		ld	l, (ix+4)
		ret

;------------------------------------------------------------------------------
; Operand berechnen und	nchsten Operator holen
;------------------------------------------------------------------------------

OPBER:		push	de
		call	evop		; Operand berechnen
		pop	de
		push	hl
		push	bc
		ld	hl, OPTAB	; Operatoren-Tabelle
		ld	bc, ARTAB-OPTAB	; Lnge	Operatoren-Tabelle
		ld	a, (iy+0)
		cpir
		jr	nz, OPBER1	; Operator nicht gefunden
		inc	iy		; Operator berlesen
OPBER1:		pop	bc
		ld	d, (hl)		; D = Prioritt aus Tabelle (C3h wenn nicht gef.)
		pop	hl
		ld	a, c
		cp	0FEh
		ret	nz		; keine Strings
		ld	a, d
		cp	18		; '+' (Stringverkettung)?
		ret	nz		; nein
		ld	d, c		; negative Prioritt fr x$ + x$
		ret

;------------------------------------------------------------------------------
; Operatoren-Tabelle - nach aufsteigender, GLEITENDER Prioritt geordnet
;------------------------------------------------------------------------------

OPTAB:		db '='
		db 0
		db '#'
		db 3
		db '<'
		db 6
		db '>'
		db 9
		db Lo(TOR) 		; OR
		db 12
		db Lo(TAND) 		; AND
		db 15
		db '+'
		db 18
		db '-'
		db 21
		db '/'
		db 24
		db '*'
		db 27
; die notw. negat. Prioritt wenn Operator nicht gefunden liefert der folgende Opcode C3h!

;------------------------------------------------------------------------------
; Arithmetik-Sprungverzweiger: Relativadresse ergibt sich aus Datenbyte OPTAB
;------------------------------------------------------------------------------

ARTAB:		jp	AR_EQ		; =
		jp	AR_NE		; #
		jp	AR_LESS		; <
		jp	AR_GRTR		; >
		jp	AR_OR		; OR
		jp	AR_AND		; AND
		jp	AR_ADD		; +
		jp	AR_SUB		; -
		jp	AR_DIV		; /
		jp	AR_MULT		; *

;------------------------------------------------------------------------------
; =
;------------------------------------------------------------------------------

AR_EQ:		call	AR_NE		; Ungleichheitstest
AR_EQ1:		ld	a, l
		xor	1		; Ergebnis negieren
		ld	l, a
		ret

;------------------------------------------------------------------------------
; #
;------------------------------------------------------------------------------

AR_NE:		call	COMP		; Vergleichs-UP
AR_NE1:		ld	hl, 0
		ld	c, 0FFh
		ret	z		; bei Gleichheit 0
		inc	l		; bei ungleich 1
		ret

;------------------------------------------------------------------------------
; <
;------------------------------------------------------------------------------

AR_LESS:	ld	a, c
		cp	0FEh
		jr	nz, AR_LESS1	; keine Strings
		call	COMP1		; Stringvergleich
		ccf
		sbc	a, a		; Cy=0 und Z=1 bei $1 > $2 bzw. 1$ == 2$
		jr	AR_NE1
AR_LESS1:	call	AR_SUB		; -
		ld	a, c
		inc	a
		jr	nz, AR_LESS2	; Ergebnis nicht integer
		bit	7, h		; Z=1 bei posit. Ergebnis (1.W >= 2.W)
		jr	AR_NE1
AR_LESS2:	bit	7, c		; dito
		jr	AR_NE1

;------------------------------------------------------------------------------
; >
;------------------------------------------------------------------------------

AR_GRTR:	call	COMP4
		jr	z, AR_NE1	; bei Gleichheit numerisch
		call	AR_LESS		; Test Op1 < Op2
		jr	AR_EQ1		; Ergebnis negieren

;------------------------------------------------------------------------------
; AND
;------------------------------------------------------------------------------

AR_AND:		ld	bc, (alh2)
		ld	a, c
		and	l
		ld	l, a
		ld	a, b
		and	h
		jr	AR_OR1

;------------------------------------------------------------------------------
; OR
;------------------------------------------------------------------------------

AR_OR:		ld	bc, (alh2)
		ld	a, c
		or	l
		ld	l, a
		ld	a, b
		or	h
AR_OR1:		ld	h, a
		ld	c, 0FFh		; Ergebnis Integer
		ret

;------------------------------------------------------------------------------
; Vergleichen
;------------------------------------------------------------------------------

COMP:		ld	a, c
		cp	0FEh
		jr	nz, COMP4	; keine Strings
COMP1:		ld	a, (acb2+1)	; Ausdruckwert "B"
		ld	c, a		; C = Lnge 2.String
		ld	de, (alh2)	; DE = Adr. 2.String
COMP2:		ld	a, (de)
		cp	(hl)
		ret	nz		; ungleich (Cy=1: 1$ > 2$; Cy=0: 1$ < 2$)
		inc	hl
		inc	de
		dec	c
		jr	z, COMP3	; Ende 2.String erreicht
		djnz	COMP2
		scf
		ccf			; Z=1, Cy=0 ---> $1 krzer/= $2
		ret			; $1 krzer $2 gilt als gleich)
COMP3:		scf			; Cy=1: vorbereiten $1 > $2
		dec	b		; Z=1 wenn identisch gleich (Cy=1)
		ret

COMP4:		ld	a, (alh2+1)	; Test Gleichheit numer.; wenn ja: Z=1
		cp	h
		ret	nz
		ld	a, (alh2)
		cp	l
		ret	nz
		ld	a, (acb2)
		cp	c
		ret	nz
		inc	a
		ret	z
		ld	a, (acb2+1)
		cp	b
		ret

;------------------------------------------------------------------------------
; -
;------------------------------------------------------------------------------

AR_SUB:		call	TSTINT
		jr	nz, AR_SUB1	; nicht beide integer
		ld	de, (alh2)	; Ausdruckwert LH 2.Operand
		sub	a
		sbc	hl, de
		ret	po		; kein berlauf
AR_SUB1:	call	TOBCD		; Integer ---> BCD
		ld	hl, asign2	; Vorzeichen
		ld	a, (hl)
		xor	80h		; Vorzeichen negieren
		ld	(hl), a
		jr	AR_ADD2		; dann addieren

;------------------------------------------------------------------------------
; +
;------------------------------------------------------------------------------

AR_ADD:		call	TSTINT
		jr	nz, AR_ADD1	; nicht beide integer
		ld	de, (alh2)	; Ausdruckwert LH 2.Operand
		sub	a
		adc	hl, de
		ret	po		; kein berlauf
AR_ADD1:	call	TOBCD		; Integer ---> BCD
AR_ADD2:	ld	ix, areg1	; Arithmetikpuffer1
		ld	a, (asign2)	; Vorzeichen 2.Operand
		and	7
		ld	c, a		; Nachkommastellen 2.Operand
		ld	a, (asign1)	; Vorzeichen 1.Operand
		and	7
		sub	c		; Nachkommast.1 - Nachkommast.2
		ld	hl, asign2	; Vorzeichen 2.Operand
		jp	p, AR_ADD3	; 1.Op hat mehr Nachkommastellen
		ld	hl, asign1	; Vorzeichen 1.Operand
		neg			; A = Differenz der Nachkommastelligkeit
AR_ADD3:	jr	z, AR_ADD5	; Nachkommastelligkeit gleich
		ld	c, a
		add	a, (hl)
		ld	(hl), a		; bisher kleinere Kommastelligkeit an grere anpassen
		dec	hl
AR_ADD4:	push	hl
		call	MUL10_1		; Wert mit bisher kleinerer Kommastelligkeit * 10
		pop	hl
		dec	c		; entsprechend Kommastelligkeitsdiff. ausfhren
		jr	nz, AR_ADD4	; Kommas untereinander bringen
AR_ADD5:	ld	b, 8
		ld	hl, alh2+1	; Ausdruckwert LH
		ld	a, (asign2)	; Vorzeichen
		cp	(ix+8)		; Vorzeichenvergleich
		jr	nz, AR_ADD7	; ungleich: Subtraktion

		sub	a		; Additionsschleife
AR_ADD6:	ld	a, (hl)
		adc	a, (ix+7)
		daa
		ld	(hl), a
		dec	hl
		dec	ix
		djnz	AR_ADD6
		jr	AR_ADD10

AR_ADD7:	sub	a		; Subtraktionsschleife
AR_ADD8:	ld	a, (hl)
		sbc	a, (ix+7)
		daa
		ld	(hl), a
		dec	hl
		dec	ix
		djnz	AR_ADD8
		jr	nc, AR_MULT8	; kein Unterlauf
		ld	b, 8
		ld	hl, asign2	; Vorzeichen
		ld	a, (hl)
		xor	80h		; Vorzeichen negieren
		ld	(hl), a
		dec	hl
		sub	a
AR_ADD9:	ld	a, 0
		sbc	a, (hl)
		daa			; Wert negieren (BCD)
		ld	(hl), a
		dec	hl
		djnz	AR_ADD9
AR_ADD10:	jr	AR_MULT8

;------------------------------------------------------------------------------
; /
;------------------------------------------------------------------------------

AR_DIV:		call	AR_MDREG
		jp	z, AERR		; A-ERROR (Division durch 0)
		ld	d, 0E8h
AR_DIV1:	call	MUL10
		ld	c, 0FFh
AR_DIV2:	call	UPDIV
		jr	nc, AR_DIV2
		ld	c, 1
		call	UPDIV
		inc	d
		jr	z, AR_DIV3
		ld	a, (acb2)	; "C" 2.Operand
		and	a
		jr	z, AR_DIV1	; wenn 0
AR_DIV3:	ld	hl, asign2	; Vorzeichen 2.Operand
		ld	a, (asign1)	; Vorzeichen 1.Operand
		sub	(hl)		; Vorz1 - Vorz2
		add	a, d		; + Zykluszhler
		bit	4, a
		jp	z, AR_NE1	; 0 bei Ergebnisunterlauf
		add	a, 10h		; Korr. Vorz. und Kommast.
		jr	AR_MULT7

;------------------------------------------------------------------------------
; *
;------------------------------------------------------------------------------

AR_MULT:	call	AR_MDREG
		jr	z, AR_MULT8	; wenn 2.Operand = 0
		ld	d, 7
		jr	AR_MULT2
AR_MULT1:	call	MUL10		; "C" 1.Operand
AR_MULT2:	ld	a, (acb1)
		and	0Fh
		ld	c, a
		jr	z, AR_MULT6	; hchste Tetrade = 0
AR_MULT3:	ld	hl, alh2+1	; "H" 2.Operand
		call	UPDIV1
		jr	nc, AR_MULT5
		ld	b, 4
AR_MULT4:	ld	a, 0
		adc	a, (hl)
		daa
		ld	(hl), a
		dec	hl
		djnz	AR_MULT4
AR_MULT5:	dec	c
		jr	nz, AR_MULT3
AR_MULT6:	dec	d
		jr	nz, AR_MULT1
		ld	a, (asign1)	; Vorzeichen 1.Operand
		ld	hl, asign2	; Vorzeichen 2.Operand
		add	a, (hl)		; beide addieren: Stellenzahl und Vorzeichen Ergebnis
AR_MULT7:	ld	(hl), a
AR_MULT8:	ld	hl, acb2	; Ausdruckwert CB
		ld	b, 4
		ld	a, (hl)
		and	0F0h		; HWT(C) = NW-Tetrade Arith.puffer
AR_MULT9:	dec	hl
		or	(hl)
		djnz	AR_MULT9
		jp	z, AR_MULT10	; wenn Arithmetikpuffer = 0
		call	DIV10		; /10, solange bis nur noch 7 signif. Tetraden
		jr	AR_MULT8

; Division /10

DIV10:		ld	b, 8
		sub	a
		ld	hl, areg2	; Arithmetik-Register
DIV10_1:	rrd
		inc	hl
		djnz	DIV10_1
		dec	(hl)		; Kommastellen runterzhlen
		ret
;
AR_MDREG:	call	TOBCD
		ld	bc, 0400h
		ld	hl, acb2	; 2.Operand
		ld	de, ZWIREG	; Arithmetik-Zwischenregister
AR_MDREG1:	ld	a, (hl)
		ld	(de), a		; 2.Operand ---> Zwischenregister
		or	c
		ld	c, a		; Nulltest vorbereiten
		ld	(hl), 0		; 2.Operand lschen
		inc	de
		inc	hl
		djnz	AR_MDREG1
		ld	a, c
		and	a		; Z=1 wenn Wert = 0
		ret

; Multiplikation *10

MUL10:		ld	hl, alh2+1	; "H" 2.Operand
		call	MUL10_1
		ld	hl, alh1+1	; "H" 1.Operand
MUL10_1:	ld	b, 8		; eigentlich Multiplikation mit 10
		sub	a
MUL10_2:	rld
		dec	hl
		djnz	MUL10_2
		ret

; Unterprogramm fr DIV

UPDIV:		ld	a, (alh2+1)	; "H" 2.Operand
		sub	c
		ld	(alh2+1), a	; Ausdruckwert "H"
		ld	hl, areg1+3	; Ende Arithmetikpuffer
		sub	a
UPDIV1:		ld	ix, ZWIREG+4	; Ende Arithmetik-Zwischenregister
		ld	b, 4		; Stellenzahl
UPDIV2:		dec	ix
		ld	a, (hl)
		bit	7, c
		jr	z, UPDIV3
		sbc	a, (ix+0)
		jr	UPDIV4
;
UPDIV3:		adc	a, (ix+0)
UPDIV4:		daa
		ld	(hl), a
		dec	hl
		djnz	UPDIV2
		ret

; Unterprogramm Integertest fr + und -

TSTINT:		ld	a, (acb2)	; A = Ausdruckwert C 2.Operand
		inc	a
		ret	nz		; nicht integer
		ld	a, (acb1)	; A = Ausdruckwert C 1.Operand
		inc	a
		ret			; Z=1 : beide Werte Integer

; Integer ---> BCD

TOBCD:		ld	hl, acb2	; 2. Operand
		call	TOBCD1
		ld	hl, acb1	; 1. Operand
TOBCD1:		ld	a, (hl)
		inc	a
		ret	nz		; nicht integer
		push	de
		ld	(hl), a		; hchstwertiges Byte ("C") = 0
		inc	hl
		inc	hl
		ld	e, (hl)
		inc	hl
		ld	d, (hl)
		inc	hl
		ex	de, hl		; HL=Integerwert, DE=Adr.Vorzeichen
		ld	a, h
		and	80h
		ld	(de), a		; Vorzeichen laden
		call	nz, AR_COMPL	; Komplement bei negativ
		push	de
		sub	a
		ld	d, a
		ld	b, 16		; Zykluszahl
TOBCD2:		add	hl, hl
		adc	a, a
		daa
		ld	e, a
		ld	a, d
		adc	a, a
		daa
		ld	d, a
		rl	c
		ld	a, e
		djnz	TOBCD2
		pop	hl		; HL = Adr. Vorzeichen
		dec	hl
		ld	(hl), e		; Zehner, Einer
		dec	hl
		ld	(hl), d		; Tausender, Hunderter
		dec	hl
		ld	(hl), c		; Zehntausender
		pop	de
		ret

; Hilfsprogramm "Aufaddieren" fr TOINT

UPTOINT:	and	0Fh
		inc	a
UPTOINT1:	dec	a
		ret	z
		add	hl, de
		jr	UPTOINT1

; Hilfsprogramm Zweierkomplement HL

AR_COMPL:	ld	a, l
		cpl
		ld	l, a
		ld	a, h
		cpl
		ld	h, a
		inc	hl
		ret

;------------------------------------------------------------------------------
; Konvertierung	von Eingabezahlen
;------------------------------------------------------------------------------

ZAHL:		ld	hl, 0		; Puffer lschen
		ld	(acb2), hl	; Ausdruckwert CB
		ld	(alh2), hl	; Ausdruckwert LH
		ld	c, 80h		; vorbereiten: Integerzahl
ZAHL1:		ld	a, (iy+0)
		cp	'.'
		jr	nz, ZAHL2	; kein Dezimalpunkt
		ld	c, 0		; Merker: Realzahl
		inc	iy
		ld	a, (iy+0)
ZAHL2:		cp	'0'
		jr	c, ZAHL5	; Ende erreicht
		cp	3Ah
		jr	c, ZAHL3	; wenn Zahl
		cp	'A'
		jr	c, ZAHL5	; Ende erreicht
		sub	7		; Korrektur fr Hex.
		cp	40h
		jr	nc, ZAHL4	; keine Hex.Tetrade
ZAHL3:		and	0Fh
		inc	c		; Kommastellenzahl erhhen
		inc	iy
		ld	b, 4		; Bytezahl
		ld	hl, alh2+1	; Pufferende
		call	MUL10_2		; *10 ---> BCD-Linksverschiebung
		jr	ZAHL1
ZAHL4:		cp	'H'-7		; Hexadezimalzahlen? (Rckkorrektur -7 von oben)
		jr	z, ZAHL6	; ja
ZAHL5:		ld	a, (alh2+1)	; "H" Ausdruckwert
		and	0Fh
		cp	0Ah
		jr	c, ZAHL7	; wenn Dezimalstelle
		call	DIV10		; BCD-Rechtsverschiebung ( /10 )
		dec	iy
		jr	ZAHL5
ZAHL6:		inc	iy		; Hexazahlen holen
		ld	c, 0FFh		; Kennung Integer
		ld	hl, alh2+1	; "H" Ausdruckwert
		ld	a, (hl)
		dec	hl
		ld	h, (hl)
		ld	l, a		; HL = Wert
		ret

ZAHL7:		sub	a
		bit	7, c
		ld	(asign2), a	; Kommastellen = 0 vorbereiten
		jr	nz, AR_MULT12	; bei Integerzahl (C >= 80h)
		ld	a, c
		ld	(asign2), a	; Kommastellen laden bei Realzahl

; Hilfsprogramm Gleitkommaarbeit

AR_MULT10:	ld	hl, asign2	; Vorzeichen
		bit	3, (hl)
		jr	nz, AR_MULT11	; Kommastellenzahl > 7
		dec	hl
		ld	a, (hl)
		and	0Fh
		jr	nz, AR_MULT12
		ld	a, (asign2)	; Vorzeichen
		and	7
		jr	z, AR_MULT12
AR_MULT11:	call	DIV10		; Div / 10 - nichtsignif. Stellen rechts weg
		jr	AR_MULT10
;
AR_MULT12:	ld	bc, (acb2)	; Ausdruckwert CB
		ld	hl, asign2	; Vorzeichen Ergebnis
		ld	a, (hl)
		and	78h
		jp	nz, AERR	; A-ERROR (berlauf)
		ld	a, (hl)
		rld
		add	a, (hl)
		and	0F0h
		add	a, c
		ld	c, a
		add	a, (hl)
		and	87h
		ld	(hl), a		; Vorzeichen laden
		ld	hl, (alh2)	; Ausdruckwert LH: CBLH jetzt geladen

; Abfrage auf BCD und ggf. Umrechnen in Integer

		ld	a, c
		and	7Fh
		ret	nz		; wenn > 1.000.000 oder real
		ld	a, 3
		cp	b
		ret	c		; RET wenn BLH > 032767
		jr	nz, TOINT
		ld	a, 27h
		cp	l
		ret	c		; dto.
		jr	nz, TOINT
		ld	a, 67h
		cp	h
		ret	c		; dto.

;------------------------------------------------------------------------------
; Umrechnen BCD ---> binr (interne Integerdarstellung)
;------------------------------------------------------------------------------

TOINT:		ld	a, b
		push	bc
		ld	c, h
		ld	b, l
		push	de
		ld	de, 10000
		ld	hl, 0
		call	UPTOINT		; Zehntausender aufaddieren
		ld	de, 1000
		ld	a, b
		rrca
		rrca
		rrca
		rrca			; NWT(A) = Tausender
		call	UPTOINT		; Tausender aufaddieren
		ld	a, b
		ld	de, 100
		call	UPTOINT		; Hunderter aufaddieren
		ld	a, c
		rrca
		rrca
		rrca
		rrca			; NWT(A) = Zehner
		ld	e, 10
		call	UPTOINT		; Zehner aufaddieren
		ld	a, c
		and	0Fh		; NWT(A) = Einer
		ld	e, a
		add	hl, de		; Einer aufaddieren
		pop	de
		pop	bc
		bit	7, c
		call	nz, AR_COMPL	; Komplement bei negativ
		ld	c, 0FFh		; Kennung: Integer
		ret

;------------------------------------------------------------------------------
; NL ausfhren,	wenn nicht Zeilenanfang
;------------------------------------------------------------------------------

NLNZA:		ld	hl, (CPOS)	; Bildschirmpointer
		dec	hl
		bit	7, (hl)
		ret	nz		; wenn Zeilenanfang
		ld	a, NL
		rst	28h		; NL
		ret


;------------------------------------------------------------------------------
; FEHLERBEHANDLUNG
;------------------------------------------------------------------------------

OERR:		ld	a, 'O'		; O-ERROR
		db  11h			; LD DE,xxxx (statt JR ERROR, spart 1 Byte)
AERR:		ld	a, 'A'          ; A-ERROR
ERROR:		push	af
		call	NLNZA		; NL ausfhren,	wenn nicht Zeilenanfang
		ld	a, '?'
		rst	28h
		ld	a, ' '
		rst	28h
		pop	af
		rst	28h		; Fehlerkennzeichen ausgeben
		ld	de, ERRTX	; " ERROR"
		call	TEXT		; allg.	Textausgabe
		push	iy
		pop	de
		ld	a, d
		cp	Hi(KONV2)	; Arbeit im Direktmodus (im Konv.-puffer)?
		jr	z, EREND	; ja: Zeile nicht ausgeben
ERROR1:		dec	de
		ld	a, (de)
		cp	NL
		jr	nz, ERROR1	; bis rckwrts Zeilenanfang gefunden
		inc	de
		call	LISTUP		; Echoausgabe konvertierte BASIC-Zeile
EREND:		jp	END


;------------------------------------------------------------------------------
; SAVE / SAVED
;------------------------------------------------------------------------------

SAVE:		ld	hl, (PBEG)	; BASIC-Programmbeginn
		ld	(KONV1), hl	; Anfang in Konvertierungspuffer1 merken
		ld	de, (PLEN)	; DE = BASIC-Programmlnge
		add	hl, de		; HL = BASIC-Programmende
		ld	a, (iy+0)
		cp	'D'             ; Kommando "saveD"?
		jr	nz, SAVE1	; nein: nur BASIC-Programm sichern
		ld	hl, (RAMTOP)	; ja: bis phys. Speicherende sichern
SAVE1:		ld	(KONV2), hl	; Ende in Konvertierungspuffer2 merken
		di
		ld	a, 27h
		out	(0F8h),	a
		ld	a, 4
		out	(0F8h),	a
		ld	a, 3
		out	(0F9h),	a	; Interrupt CTC1 sperren - keine BS-Ausgabe
		ld	bc, 512		; langen Kennton vorbereiten (512 * AAh)
SAVE2:		ld	d, 0AAh
		push	bc
		call	SBYT		; Save Byte: Kennton ausgeben
		pop	bc
		dec	c
		jr	nz, SAVE2
		djnz	SAVE2		; bis Ende Kennton
		ld	bc, (KONV1)	; akt. Anfangsadresse
		sub	a
		sbc	hl, bc		; akt. Anfang > Endadresse ?
		jr	c, SAVE4	; wenn ja: Datenende
		ld	b, l
		inc	b		; B = Blocklnge
		ld	d, 0E6h		; Anfangskennung
		call	SBYT
		ld	c, 0		; Prfsummenzhler in C
		ld	hl, (KONV1)	; akt. Anfangsadresse
		ld	d, b
		call	SBYT		; 1.Byte = Blocklnge (1 bis 00H=256)
		ld	d, h
		call	SBYT		; 2.Byte = H(Anfangsadr. Block)
		ld	d, l
		call	SBYT		; 3.Byte = L(Anfangsadr. Block)
		call	SBYT0		; 4.Byte = 0
SAVE3:		ld	d, (hl)		; Datenblock ausgeben
		call	SBYT
		inc	hl
		djnz	SAVE3		; entsprechend Blocklnge wiederholen
		ld	a, c
		neg			; Prfsumme negieren ---> Quersumme Block & PS = 0
		ld	d, a
		call	SBYT		; Prfsumme ausgeben
		ld	(KONV1), hl	; akt. Anfang: nchstes Byte nach Block
		ld	hl, (KONV2)	; HL = Endadresse
		ld	bc, 0110h	; kurzer Synchronton (16 * AAh)
		jr	SAVE2		; nchsten Block ausgeben

;------------------------------------------------------------------------------
; physische Magnetband-Ausgabe - D=Zeichen; C=Prfsummenzhler
;------------------------------------------------------------------------------

; Save Null-Byte

SBYT0:		ld	d, 0

; Save Byte

SBYT:		push	hl
		ld	h, 8		; Zykluszahl
SBYT1:		ld	a, e		; vorheriger Bitstand
		rlc	d		; Zeichen links verschieben
		xor	d
		and	1		; nderung im letzten Bit?
		ld	l, 8
		jr	nz, SBYT2	; ja: lange Flanke
		ld	l, 4		; nein: zwei kurze Flanken
		call	SBIT
SBYT2:		call	SBIT
		ld	e, d		; E = letzter Bitstand
		dec	h
		jr	nz, SBYT1	; bis alle 8 Bits durch
		ld	a, d
		add	a, c		; Prfsumme korrigieren
		ld	c, a
		pop	hl
		ret

; Save Bit (Flankenausgabe)

SBIT:		in	a, (0F8h)
		cp	2
		jr	nz, SBIT	; bis Kanal CTC0 = 2
		ld	a, 25h
		out	(0F8h),	a
		ld	a, l		; neue Konstante (4 oder 8) programmieren
		out	(0F8h),	a
SBIT1:		in	a, (0F8h)
		dec	a
		jr	nz, SBIT1	; bis Kanal CTC0 = 1
		ret

; Endblock ausgeben (Ende SAVE-Routine)

SAVE4:		ld	d, 0E6h
		call	SBYT		; Anfangskennung Endeblock E6h
		call	SBYT0		; Blocklnge = 0
		call	SBYT0		; H(Adresse) = 0
		ld	hl, (PLEN)	; BASIC-Programmlnge
		ld	d, l
		call	SBYT		; L(Programmlnge)
		ld	d, h
		call	SBYT		; H(Programmlnge)
		ld	d, 0AAh
		call	SBYT		; Endekennung AAh
		rst	0		; CTC neu initialisieren (wie RESET)


;------------------------------------------------------------------------------
; LOAD
;------------------------------------------------------------------------------

; Load-Hilfsprogramm

LOAD0:		ld	d, 0FFh		; Merker "Anfangskennung E6h suchen"
		ld	c, 0		; Prfsummenzhler
		call	LBYT
		ld	b, a		; B = 1.Byte (Blocklnge)
		call	LBYT
		ld	h, a		; H = 2.Byte (H-Adresse)
		call	LBYT
		ld	l, a		; L = 3.Byte (L-Adresse)
		ret

; eigentliche LOAD-Routine

LOAD:		di
		call	LOAD0		; Blockbeschreiber holen
		ld	(alh2), hl	; ursprngliche Quelladresse merken
LOAD1:		push	de		; in der Schleife: HL = urspr. Adr. aus Blockbeschr.
		ld	de, (alh2)	; DE = ursprngliche Quelladresse
		or	a
		sbc	hl, de		; HL = Displacement (bisher vom MB gelesene Bytes)
		ld	de, (PBEG)	; BASIC-Programmbeginn
		add	hl, de		; + Displacement ---> HL = akt. Zieladresse
		pop	de
		call	LBYT		; 4.Byte (Typ) = 0 ?
		and	a
		jr	nz, TERR	; T-ERROR, wenn nicht
LOAD2:		call	LBYT
		ld	(hl), a		; Daten einlesen
		inc	hl
		djnz	LOAD2		; gem Blocklnge in B
		call	LBYT		; Prfsumme lesen (C=0 wenn korrekt)
		sub	a
		add	a, c
		jr	z, LOAD3	; wenn richtige Prfsumme
		ld	de, RECTX	; "RECORD ERROR"
		call	TEXT		; allg.	Textausgabe
TERR:		ld	IYU, Hi(KONV2)	; LD IYH,H(KONV2): keine Echoausgabe in Fehlerroutine
		ei
		ld	a, 'T'
		jp	ERROR		; "? T ERROR"
LOAD3:		call	LOAD0		; nchsten Blockbeschreiber holen
		ld	a, h
		and	a
		jr	nz, LOAD1	; wenn H-Adresse <> 0 : weiterer Block
		call	LBYT		; L = L(Programmlnge)
		ld	h, a		; HL = Programmlnge
		ld	(PLEN),	hl	; ablegen
		ei
		ld	de, (PBEG)	; DE = Adresse 1. BASIC-Zeile
		call	LISTUP		; Echoausgabe 1. BASIC-Zeile zur Kontrolle
		jp	MEM		; Ermitteln freie Bytes; Interpreterstart

; Hilfsprogramm physische MB-Eingabe: 1 Byte laden

LBYT:		push	bc
		ld	c, 0		; Ergebnisbyte vorbereiten
		ld	a, (1000h)
		and	80h
		ld	e, a		; E = Anfangs-Eingangsstatus
LBYT1:		ld	a, c
		and	7Fh		; nur niedere 7 Bit
		rlca			; nach links verschieben
		ld	c, a		; C = b6-b5-b4-b3-b2-b1-b0-0
LBYT2:		ld	a, (1000h)
		and	80h
		cp	e
		jr	z, LBYT2	; bis Flanke gefunden
		rlca			; Bit 0 ist Status nach Flanke
		or	c
		ld	c, a		; C wird gelesenes Byte
		ld	b, 114
LBYT3:		djnz	LBYT3		; Zeitschleife
		ld	a, (1000h)
		and	80h
		ld	e, a		; E = Status nach 1 Abtasttakt entspr. LANGER Flanke
		bit	7, d
		jr	z, LBYT6	; Anfangskennung bereits erkannt
		ld	a, c
		cp	0E6h		; Anfangskennung gefunden?
		jr	nz, LBYT4	; nein
		xor	a		; A = 0 ---> Zeichen nicht zu negieren
		jr	LBYT5
LBYT4:		cp	19h		; negierte Anfangskennung?
		jr	nz, LBYT1	; weitersuchen wenn weder/noch
		ld	a, 0FFh		; A = FFh ---> Zeichen negieren
LBYT5:		ld	(ezlen), a	; Negierungskenner ablegen
		ld	d, 9		; Zykluszahl 1 Byte Daten
LBYT6:		dec	d
		jr	nz, LBYT1	; bis Byte gelesen
		ld	d, 8		; Zykluszahl fr nchstes Einlesen
		ld	a, (ezlen)	; Negierungskenner
		xor	c		; ggf. negieren
		pop	bc
		ld	e, a
		add	a, c
		ld	c, a		; Prfsumme in C aufaddieren
		ld	a, e		; A = gelesenes Zeichen
		ret			; zugleich Anfangskennung SWTAB (mu >C0h sein)


;------------------------------------------------------------------------------
; Schlsselworttabelle
;------------------------------------------------------------------------------

SWTAB:		db "END"
		db Lo(TEND)
		db "REM"
		db Lo(TREM)
		db "GOSUB"
		db Lo(TGOS)
		db "CLS"
		db Lo(TCLS)
		db "IF"
		db Lo(TIF)
		db "THEN"
		db Lo(TTHEN)
		db "INPUT"
		db Lo(TINP)
		db "PRINT"
		db Lo(TPRI)
		db '?'
		db Lo(TPRI)
		db "RETURN"
		db Lo(TRET)
		db "LOAD"
		db Lo(TLOAD)
		db "LET"
		db Lo(TLET)
		db "SAVE"
		db Lo(TSAVE)
		db "GOTO"
		db Lo(TGOTO)
		db "CALL"
		db Lo(TCALL)
		db "IN("
		db Lo(TIN)
		db "OUT"
		db Lo(TOUT)
		db "RUN"
		db Lo(TRUN)
		db "LIST"
		db Lo(TLIST)
		db "DIM"
		db Lo(TDIM)
		db "USR("
		db Lo(TUSR)
		db "PLOT"
		db Lo(TPLOT)
		db "UNPLOT"
		db Lo(TUNP)
		db "FOR"
		db Lo(TFOR)
		db "TO"
		db Lo(TTO)
		db "STEP"
		db Lo(TSTEP)
		db "NEXT"
		db Lo(TNEXT)
		db "ASC("
		db Lo(TASC)
		db "CHR$("
		db Lo(TCHR)
		db "INKEY$"
		db Lo(TINKEY)
		db "DATA"
		db Lo(TDATA)
		db "NEW"
		db Lo(TNEW)
		db "PEEK("
		db Lo(TPEEK)
		db "POKE"
		db Lo(TPOKE)
		db "RND("
		db Lo(TRND)
		db "OR"
		db Lo(TOR)
		db "AND"
		db Lo(TAND)
		db "READ"
		db Lo(TREAD)
		db "RESTORE"
		db Lo(TREST)
		db "INT("
		db Lo(TINT)
		db "LEN("
		db Lo(TLEN)
		db "RANDOMIZE"
		db Lo(TRAND)


;------------------------------------------------------------------------------
; Tokentabelle
;------------------------------------------------------------------------------

		ORG 0FCAh		; Adresse wegen L-Teil der Token zwingend fest!

TEND:		dw END
TREM:		dw REM
TGOS:		dw GOSUB
TCLS:		dw CLS
TIF:		dw IF
TINP:		dw INPUT
TPRI:		dw PRINT
TREST:		dw RESTO
TRET:		dw RETRN
TGOTO:		dw GOTO
TCALL:		dw CALL
TRUN:		dw RUN
TLIST:		dw LIST
TLET:		dw LET
TLOAD:		dw LOAD
TSAVE:		dw SAVE
TPOKE:		dw POKE
TOUT:		dw OUT
TFOR:		dw FOR
TNEXT:		dw NEXT
TDIM:		dw DIM
TPLOT:		dw PLOT
TUNP:		dw UNPLT
TNEW:		dw NEW
TRAND:		dw RANDO
TREAD:		dw READ
TDATA:		dw REM			; DATA

; Funktionstoken
TTO:		equ TEND-1		; TO
TSTEP:		equ TEND-2		; STEP
TTHEN:		equ TEND-3		; THEN
TAND:		equ TEND-4		; AND
TOR:		equ TEND-5		; OR
TASC:		equ TEND-6		; ASC
TCHR:		equ TEND-7		; CHR$
TIN:		equ TEND-8		; IN
TINKEY:		equ TEND-9		; INKEY$
TINT:		equ TEND-10		; INT
TLEN:		equ TEND-11		; LEN
TPEEK:		equ TEND-12		; PEEK
TRND:		equ TEND-13		; RND
TUSR:		equ TEND-14		; USR


;------------------------------------------------------------------------------
; Arbeitsspeicher
;------------------------------------------------------------------------------

		org RAM

PBEG:		ds 2			; BASIC-Programmbeginn
PLEN:		ds 2			; BASIC-Programmlnge
RAMTOP:		ds 2			; phys.	Speicherende
ZZ:		ds 2			; Zeilenzahl Bildschirm
CPOS:		ds 2			; Bildschirmpointer
ezlen:		ds 2			; Lnge	Eingabezeile u.. (univ. Merkzelle)
arnd:		ds 2			; RND-Merkzelle

areg2:		ds 4			; Arithmetik-Puffer2  )
acb2:		ds 2			; Ausdruckwert CB     ) 2. Operand bzw.
alh2:		ds 2			; Ausdruckwert LH     ) Ergebnis
asign2:		ds 1			; Vorzeichen          )

areg1:		ds 4			; Arithmetik-Puffer1  )
acb1:		ds 2			; Ausdruckwert CB     ) 1. Operand
alh1:		ds 2			; Ausdruckwert LH     )
asign1:		ds 1			; Vorzeichen          )

ZWIREG:		ds 4			; Zwischenregister 2.Operand bei Mult./Div.

BZ:		ds 2			; Anfang Eingabezeile BWS
READPT:		ds 2			; READ-Pointer
CHRMZ:		ds 1			; Merkzelle fr	CHR$
TACOD:		ds 1			; Tastencode bei INKEY$

KONV1:		ds 2			; Konvertierungspuffer Zeilennummer
KONV2:		ds 136			; Konvertierungspuffer akt. Zeile,
					; anschlieend Stackbereich (ca. 100 Byte)

BWS:		ds 844			; Beginn Bildwiederholspeicher
					; max. 28 Zeilen a 30 Zeichen (bei 2,5 MHz Takt)
					; od. max. 20 Z. a 41 Zeichen (bei 3,5 MHz Takt)
					; + je 1 Byte f. RST30H = 841 bzw. 821 Byte
					; anschlieend RAM: BASIC-Programm
					; und Variablen- bzw. Feldspeicher

; fr BWS-Adress-Kompatibilitt zu Version 3.2 (dann 21 BS-Zeilen bei 3,5 MHz mglich)

;KONV2:		ds 116			; Konvertierungspuffer akt. Zeile
;BWS:		ds 864			; Beginn Bildwiederholspeicher


		end

